/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Thu May 30 17:37:02 2002 by Jeff Dalton
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 */

package ix.ichat;

import javax.swing.*;

import java.awt.Container;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.*;

import java.util.*;

import ix.icore.*;

import ix.iface.util.TranscriptTextArea;
import ix.iface.util.Reporting;
import ix.iface.util.ComboChoice;
import ix.iface.util.IconImage;
import ix.iface.util.CatchingActionListener;

import ix.itest.SendPanel;

import ix.ip2.WaitForReportsActivity;

import ix.util.*;
import ix.util.lisp.*;

/**
 * A frame for chat dialogues.
 */
public class ChatFrame extends JFrame implements ActionListener {

    protected Container contentPane = getContentPane();

    IXAgent agent;

    TranscriptPanel transcriptPanel = new TranscriptPanel();
    SendPanel sendPanel;

    public ChatFrame(IXAgent agent, String title) {
	super(title);
	this.agent = agent;

	setIconImage(IconImage.getIconImage(this));

	sendPanel = new ChatSendPanel(agent);

	setJMenuBar(makeMenuBar());
	contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
	contentPane.add(transcriptPanel);
	contentPane.add(sendPanel);
	pack();
    }

    /**
     * Creates the menu bar
     */
    protected JMenuBar makeMenuBar() {
	JMenuBar bar = new JMenuBar();

	JMenu fileMenu = new JMenu("File");
	bar.add(fileMenu);
	if (agent instanceof IChat)
	    // Standalone I-Chat
	    fileMenu.add(makeMenuItem("Exit"));
	else
	    // Part of another agent
	    fileMenu.add(makeMenuItem("Close"));

	return bar;
    }

    // /\/: Must find a better way to do this rather than have
    // this method everywhere.
    protected JMenuItem makeMenuItem(String text) {
	JMenuItem item = new JMenuItem(text);
	item.addActionListener(CatchingActionListener.listener(this));
	return item;
    }

    /**
     * Action interpreter
     */
    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	Debug.noteln("ChatFrame action:", command);
	if (command.equals("Close")) {
	    setVisible(false);
	}
	else if (command.equals("Exit")) {
	    System.exit(0);
	}
	else
	    Debug.noteln("Nothing to do for", command);
    }

    public void newMessage(ChatMessage message) {
	transcriptPanel.newMessage(message);
	// sendPanel.noteSender(message);
    }

    public void initReport(TaskItem item) {
	// Called to set up to report back about an item.
	sendPanel.initReport(item);
    }

    public void sendCopy(String destination, Sendable contents) {
	sendPanel.sendCopy(destination, contents);
    }

    class TranscriptPanel extends JPanel {

	TranscriptTextArea transcript = new TranscriptTextArea(15, 40);

	TranscriptPanel() {
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    setBorder(BorderFactory.createTitledBorder("Transcript"));
	    transcript.setEditable(false);
	    add(new JScrollPane(transcript));
	}

	public void newMessage(ChatMessage message) {
	    transcript.appendLine(message.getSenderId()
				  + " at " + Reporting.dateString()
				  + ":");
	    transcript.appendIndentedLines(3, message.getText());
	}

    }

    /**
     * A panel for sending issues, activities, reports, and chat messages.
     */
    class ChatSendPanel extends SendPanel {

	ChatSendPanel(IXAgent agent) {
	    super(agent);
	}

	protected ChatMessage messageFromControls() {
	    ChatMessage message = super.messageFromControls();
	    // Record our own outgoing messages in the transcript.
	    ChatFrame.this.transcriptPanel.newMessage(message);
	    return message;
	}

	public void send(String destination, Sendable contents) {
	    // N.B. Can modify contents.
	    if (contents instanceof Issue || contents instanceof Activity) {
		TaskItem item = (TaskItem)contents;
		if (!destination.equalsIgnoreCase("me")
		      && item.getReportBack() == YesNo.YES
		      && item.getRef() == null) {
		    // Create an activity that will record any reports-back.
		    Activity activity =
			// /\/: might want a more convenient constructor
			new WaitForReportsActivity
			    (destination, item.getPattern());
		    item.setRef(activity.getId());
		    activity.setPriority(item.getPriority());
		    agent.handleNewActivity(activity);
		}
	    }
	    super.send(destination, contents);
	}

    }

}
