/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Sat May  4 17:00:56 2002 by Jeff Dalton
 * Copyright: (c) 2001, 2002, AIAI, University of Edinburgh
 */

package ix.icore;

import java.util.*;

import ix.icore.domain.Refinement;
import ix.util.*;
import ix.util.lisp.*;

public class Variable implements Comparable {

    Object name;		// must be Comparable
    Object value;

    // Info and debugging fields
    static List allVariables = new LListCollector();
    static int count = 0;
    public int number = ++count;
    // public PNode sourceNode;
    public Refinement sourceRefinement;

    public Variable(Object name) {
	this.name = name;
	allVariables.add(this);
    }

    public Object getName() { return name; }

    public Object getValue() { return value; }

    public void setValue (Object v) {
	Debug.noteln("Setting value of " + this + " to " + v);
	Debug.assert(!(v instanceof ItemVar));
	Debug.assert(!(v instanceof Variable));
	this.value = v;
    }

    public int compareTo(Object o) {
	if (o instanceof Variable)
	    return ((Comparable)name).compareTo(((Variable)o).name);
	else
	    throw new ClassCastException
		("Cannot compare Variable " + this + " to " + o);
    }

    public String displayString() {
	return "[" + number + ":" + name
	    + (value == null ? "" : "=" + value)
	    + "]";
    }

    public String toString() {
	return displayString();
    }

    /* Static utilities */

    public static Set varsIn(Object tree) {
	if (tree instanceof Variable) {
	    HashSet result = new HashSet();
	    result.add(tree);
	    return result;
	}
	else if (tree instanceof LList)
	    return varsIn((LList)tree);
	else
	    return new HashSet();
    }

    public static Set varsIn(LList tree) {
	final Set vars = new HashSet();
	tree.walkTree(new Function1() {
	    public Object funcall(Object item) {
		if (item instanceof Variable)
		    vars.add(item);
		return null;
	    }
	});
	return vars;
    }

    public static Set unboundVarsIn(Collection c) {
	Set vars = new HashSet();
	for (Iterator i = c.iterator(); i.hasNext();) {
	    Variable v = (Variable)i.next();
	    if (v.getValue() == null)
		vars.add(v);
	}
	return vars;
    }

    public static boolean isFullyBound(Object tree) {
	if (tree instanceof Variable)
	    return ((Variable)tree).getValue() != null;
	else if (tree instanceof Cons) {
	    Cons c = (Cons)tree;
	    return isFullyBound(c.car()) && isFullyBound(c.cdr());
	}
	else
	    return true;
    }

    public static Object removeVars(Object tree) {
	if (tree instanceof Variable)
	    return ((Variable)tree).getName();
	else if (tree instanceof Cons) {
	    Cons t = (Cons)tree;
	    return new Cons(removeVars(t.car()), (LList)removeVars(t.cdr()));
	}
	else
	    return tree;
    }

}
