/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Wed May  1 01:12:21 2002 by Jeff Dalton
 * Copyright: (c) 2002, AIAI, University of Edinburgh
 */

package ix.icore.domain;

import java.lang.reflect.Method;
import java.io.Serializable;
import java.util.*;

import ix.icore.Sendable;

import ix.util.*;
import ix.util.lisp.*;
import ix.util.match.MatchEnv;

public class Constraint implements Serializable, Cloneable, Sendable {

    protected Symbol type;
    protected Symbol relation;
    protected List parameters;

    // For SendAble interface /\/
    protected Name senderId;	// source agent name, if source is external

    public Constraint() {
    }

    public Constraint(Symbol type, Symbol relation, List parameters) {
	this.type = type;
	this.relation = relation;
	this.parameters = parameters;
    }

    public Symbol getType() { return type; }
    public void setType(Symbol type) { this.type = type; }

    public Symbol getRelation() { return relation; }
    public void setRelation(Symbol relation) { this.relation = relation; }

    public List getParameters() { return parameters; }
    public void setParameters(List p) { this.parameters = p; }

    public Object getParameter(int i) { return parameters.get(i); }

    public Name getSenderId() { return senderId; }
    public void setSenderId(Name id) { this.senderId = id; }

    public Constraint instantiate(MatchEnv env, Function1 ifUnbound) {
//  	throw new RuntimeException
//  	    ("Don't know how to instantiate a constraint yet.");

	Constraint result = new Constraint(type, relation, null);
	if (parameters != null) {
	    Class[] sig = new Class[]{MatchEnv.class, Function1.class};
	    Object[] args = new Object[]{env, ifUnbound};
	    result.parameters = LList.newLList(parameters); // copy
	    for (ListIterator i = result.parameters.listIterator();
		 i.hasNext();) {
		Object p = i.next();
		Method m = Fn.getMethod(p, "instantiate", sig);
		if (m != null)
		    i.set(Fn.apply(p, m, args));
		else
		    // Handle LLists and lone item-vars.
		    // /\/: Need something more general.
		    i.set(env.instantiateTree(p, ifUnbound));
	    }
	}
	return result;
    }

    public Object clone() throws CloneNotSupportedException {
	Constraint copy = (Constraint)super.clone();
	return copy;
    }

    public String toString() {
	return "Constraint[" + type + ", " + relation + ", "
	                     + parameters + "]";
    }

}

// Issues:
// * Should this class be in icore rather than icore.domain?
// * Should it really be Sendable?
