/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Tue Apr 16 13:49:05 2002 by Jeff Dalton
 * Copyright: (c) 2002, AIAI, University of Edinburgh
 */

package ix.icore.domain;

import java.io.Serializable;
import java.util.*;

import ix.icore.Variable;

import ix.util.*;
import ix.util.lisp.*;
import ix.util.match.MatchEnv;

public class PatternAssignment implements Serializable {

    protected LList pattern;
    protected Object value;

    public PatternAssignment() {
    }

    public PatternAssignment(LList pattern, Object value) {
	this.pattern = pattern;
	this.value = value;
    }

    public LList getPattern() {	return pattern; }
    public void setPattern(LList pattern) { this.pattern = pattern; }

    public Object getValue() { return value; }
    public void setValue(Object value) { this.value = value; }

    public PatternAssignment instantiate(MatchEnv env, Function1 ifUnbound) {
	LList ipat = (LList)env.instantiateTree(pattern, ifUnbound);
	Object ival = env.instantiateTree(value, ifUnbound);
	return new PatternAssignment(ipat, ival);
    }

    public Set getVariables() {
	// N.B. should not be called until after instantiation. /\/
	Set vars = Variable.varsIn(pattern);
	vars.addAll(Variable.varsIn(value));
	return vars;
    }

    public String toString() {
	return "p=v[" + pattern + "=" + value + "]";
    }

}
