/****************************************************************************
 * A frame with a menu bar to close it, and a scrollable text 
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iface.ui;

import ix.*;
import ix.util.*;
import ix.iface.ui.util.*;

import java.util.*;
import java.awt.*;
import java.net.URL;
import java.io.IOException;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A frame with a menu bar to close it, and a scrollable text 
 */
public class AboutFrame extends JFrame 
  implements ActionListener// , HyperlinkListener 
{
  
  private JEditorPane textPane = new JEditorPane();
  private URL original;

  /**
   * Puts up a frame with a menu bar and a scrollable text area that can
   * display HTML (including images), follow links when clicked, and select
   * text, but NOT edit.
   */
  private AboutFrame(){
    super();
    getContentPane().setLayout(new BorderLayout(0,0));
    setSize(550,650);

    JMenuBar menuBar = new JMenuBar();
    setJMenuBar(menuBar);
    JMenu fileMenu = new JMenu("File");	 
    menuBar.add(fileMenu);
    JMenuItem item = new JMenuItem("Close");	 
    item.setActionCommand("close");
    item.addActionListener(this);
    fileMenu.add(item);

    /*
    JScrollPane scrollPane = new JScrollPane();
    getContentPane().add(BorderLayout.CENTER, scrollPane);
    scrollPane.setBounds(0,0,400,300);
    mainPanel = new JPanel();
    scrollPane.getViewport().add(mainPanel); 
    mainPanel.add(htmlPane);
    */

    textPane.setEditable(false);
    //textPane.addHyperlinkListener(this);
    JScrollPane textView = new JScrollPane(textPane);
    getContentPane().add(BorderLayout.CENTER, textView);

    setTitle("About");
  }

  /**
   * As above but the given URL is displayed and remembered as the original
   * page.
   */
  public AboutFrame(ImageIcon icon, String initText){
    this();	
    displayIconText(icon, initText);
  }
  /**
   * As above but the given file name is converted to a URL using the text
   * area of the resources. 
   * An example call is <code> AboutFrame("ide-help.html") <\code>
   */
  public AboutFrame(String initText){
    this();	
    try {
      //ImageIcon icon = Util.resourceImageIcon(fileName);
      //displayIconText(icon, initText);
      displayText(initText);
    }
    catch (Exception e) {
      Debug.noteException(e);
      displayText(initText);
    }
  }
  	

  /**
   * Displays the given URL in the text area, but does not remember it as the 
   * original page.
   */
  public void displayIconText(ImageIcon icon, String starter) {
    displayText(starter);
  }
  private void displayText(String starter) {
    String text = starter + "\n" +   //newline + 
      "I-X version " + ix.Release.version + ", " + ix.Release.date;
    textPane.setText(text);
  }


  public void actionPerformed(java.awt.event.ActionEvent event) {
    Object object = event.getSource();
    String command = ((AbstractButton)object).getActionCommand();
    //Debug.noteln("action object is",object);
    Debug.noteln("AboutFrame: action command is",command);
    if (command == "close") this.setVisible(false);
    else {
      String txt = ((AbstractButton)object).getText();
      if ((txt == "") || txt == null) txt = command;
      UIUtil.notImplemented(this, txt);
    }
  }
}
/*
 * Document all public methods
 */
