/****************************************************************************
 * A toolbar button with text and icon that is not necessarily shown
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iface.ui;

import ix.*;
import ix.util.*;
import ix.iface.ui.util.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
//import java.net.*;

/**
 * A toolbar button that is not necessarily shown
 *
 */
public class IXToolButton extends Object
{

  String command;
  String label;
  Icon icon;
  boolean shown;
  char mnemo = KeyEvent.VK_UNDEFINED;
  String toolTip;
  static final char noMnemo = KeyEvent.VK_UNDEFINED;

  JButton button;
  JToolBar bar;
  ActionListener al;

  /**
   * Creates an object that holds the information (label, icon, toolTip).
   * Does not create button.
   */
  public IXToolButton(JToolBar bar, ActionListener al, 
		      String command, String label, Icon icon, String toolTip){
    super();
    this.bar = bar;
    this.al = al;
    this.command = command;
    this.label = label;
    this.icon = icon;
    this.toolTip = toolTip;
  }
  /**
   * As above with mnemo.
   */
  public IXToolButton(JToolBar bar, ActionListener al, String command, 
		      String label, Icon icon, char mnemo, String toolTip){
    this(bar, al, command, label, icon, toolTip);
    this.mnemo = mnemo;
  }
  /**
   * Creates a button (if necessary) and shows it on its toolbar.
   */
  public void show() {
    if (button == null) {
      button = new JButton();
      button.setVerticalTextPosition(AbstractButton.BOTTOM);
      button.setHorizontalTextPosition(AbstractButton.CENTER);
    }
    if (icon != null) button.setIcon(icon);
    if (label != null) button.setText(label);
    if (mnemo != noMnemo) button.setMnemonic(mnemo);
    if (toolTip != null) button.setToolTipText(toolTip);
    if (command != null) button.setActionCommand(command);
    if (al != null) button.addActionListener(al);
    if (bar != null) {
      bar.add(button);
      shown = true;
    }
  }
  /**
   * hides the button from its toolbar.
   */
  public void hide(){
    bar.remove(this.button);
    shown = false;
  }
  public void showText() {
    if (button != null) button.setText(label);
  }
  public void hideText() {
    if (button != null) button.setText("");
  }
  
}
