/****************************************************************************
 * A frame with a menu bar, a tool bar, file/about/exit dialogues.
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iface.ui;

import ix.*;
import ix.util.*;
import ix.iface.ui.util.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
//import java.net.*;

/**
 * A frame with a menu bar, a tool bar, and file/about/exit dialogues.
 *
 * Contains a method to create a default menu/tool bar and useful methods for
 * building menu and tool bars.
 * Creates an About dialog, file dialogs (open, save as), provides default 
 * functions for openFile, saveFileAs, toggleToolbar, exit, and aboutDialog.
 */
public class JFrameMT extends javax.swing.JFrame implements ActionListener
{
  /**
   * Creates a frame with a menu bar, a tool bar, a main panel.
   * Also creates required dialogs (open/save file, about, exit) and
   * handles layout.
   */
  public JFrameMT(){
    super();
    makeFrame();    
    makeMainPanel();    
    //{{REGISTER_LISTENERS
    //the frame itself is an action listener which is added to all its 
    // menu and toolbar items as they are made
    //}}
  }
  /**
   * Creates a frame (see JFrameMT()) with a given title
   * @param title a string representing the frame's title
   */
  public JFrameMT(String title){
    this();	
    setTitle(title);
  }
  	
  public void makeFrame() {
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    getContentPane().setLayout(new BorderLayout(0,0));
    setSize(488,309);
    setVisible(false);
  }


  public String imageDir = "images/";
  //some sizes for toolbar buttons
  int TBB_Y = 4;
  int TBB_WIDTH = 51;
  int TBB_HEIGHT = 27;
  int TB_HEIGHT = TBB_HEIGHT+6;

  //{{DECLARE_CONTROLS
  // dialogs--------------
  public JAboutDialog aboutDialog;
  public java.awt.FileDialog saveFileDialog;
  public java.awt.FileDialog openFileDialog;
  
  // main panel for application stuff
  public javax.swing.JPanel mainJPanel = new javax.swing.JPanel();

  // toolbar--------------
  public JPanel JPanel2 = new javax.swing.JPanel();
  // menu bar-------------
  public JMenuBar menuBar = new javax.swing.JMenuBar();
  public JToolBar toolBar = new javax.swing.JToolBar();
  public JMenu fileMenu = makeMenu(menuBar, "File");
  public JMenu editMenu = makeMenu(menuBar, "Edit");
  public JMenu viewMenu = makeMenu(menuBar, "View");
  public JMenu toolsMenu = makeMenu(menuBar, "Tools");
  public JMenu helpMenu = makeMenu(menuBar, "Help");
  public JMenu optionsMenu;

  
  protected ImageIcon newIcon = Util.resourceImageIcon("ix-new.gif");
  protected ImageIcon openIcon = Util.resourceImageIcon("ix-open.gif");
  protected ImageIcon saveIcon = Util.resourceImageIcon("ix-save.gif");
  protected ImageIcon revertIcon = Util.resourceImageIcon("ix-revert.gif");
  protected ImageIcon cutIcon = Util.resourceImageIcon("ix-cut.gif");
  protected ImageIcon copyIcon = Util.resourceImageIcon("ix-copy.gif");
  protected ImageIcon pasteIcon = Util.resourceImageIcon("ix-paste.gif");
  protected ImageIcon helpIcon = Util.resourceImageIcon("ix-help.gif");

  protected IXToolItem newItem = 
    new IXToolItem(fileMenu, toolBar, this, "new", "New", newIcon, 'N', 
		   "Create new item");
  protected IXToolItem openItem =
    new IXToolItem(fileMenu, toolBar, this, "open", "Open...", "Open", 
		   openIcon, 'O', "Open file and load");
  protected IXToolItem saveItem =
    new IXToolItem(fileMenu, toolBar, this, "save", "Save", saveIcon, 'S',
		   "Save to file");
  protected IXToolItem saveAsItem =
    new IXToolItem(fileMenu, this, "saveAs", "Save as...", 
		   "Save to new file");

  protected IXToolItem closeItem =
    new IXToolItem(fileMenu, this, "close", "Close window", 'C',
		   "Close window");
  protected IXToolItem exitItem =
    new IXToolItem(fileMenu, this, "exit", "Exit application", 'X',
		   "Exit application");


  protected IXToolItem revertItem =
    new IXToolItem(editMenu, toolBar, this, "revert", "Undo", revertIcon,'U',
		   "Undo (revert changes)");
  protected IXToolItem cutItem =
    new IXToolItem(editMenu, toolBar, this, "cut", "Cut", cutIcon, 'C',
		   "Cut");
  protected IXToolItem copyItem =
    new IXToolItem(editMenu, toolBar, this, "copy", "Copy", copyIcon, 'C',
		   "Copy");
  protected IXToolItem pasteItem =
    new IXToolItem(editMenu, toolBar, this, "paste", "Paste", pasteIcon, 'P',
		   "Paste");
  protected IXToolItem helpItem =
    new IXToolItem(helpMenu, toolBar, this, "help", "Help", helpIcon, 'H',
		   "Help");
  protected IXToolItem manualItem =
    new IXToolItem(helpMenu, this, "manual", "Manual", 'M',
		   "Manual");
  protected IXToolItem aboutItem =
    new IXToolItem(helpMenu, this, "about", "About", 'A',
		   "About the application");
  



  private void makeMainPanel() {
    //mainJPanel.setLayout(null);   //works but won't resize components
    //mainJPanel.setLayout(new BorderLayout(0,0)); //works for domed not other
    mainJPanel.setLayout(new GridBagLayout());
    getContentPane().add(BorderLayout.CENTER, mainJPanel);
    //mainJPanel.setBounds(0,this.TB_HEIGHT,488,309-this.TB_HEIGHT);
  } 
  
  public void makeMenuToolBars(){
    //{{INIT_CONTROLS
    setJMenuBar(menuBar);
    // file dialogs
    if (saveFileDialog == null) saveFileDialog = new java.awt.FileDialog(this);
    saveFileDialog.setMode(FileDialog.SAVE);
    saveFileDialog.setTitle("Save");
    if (openFileDialog == null) openFileDialog = new java.awt.FileDialog(this);
    openFileDialog.setMode(FileDialog.LOAD);
    openFileDialog.setTitle("Open");

    // toolbar
    JPanel2.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
    getContentPane().add(BorderLayout.NORTH, JPanel2);
    //JPanel2.setBounds(0,0,488,TB_HEIGHT);
    toolBar.setAlignmentY(0.222222F);
    JPanel2.add(toolBar);
    //toolBar.setBounds(0,0,395,TB_HEIGHT);
  }


  /**
   * Makes a default menubar and toolbar.
   * Menus: File, Edit, View, Help
   */
  /*
  public void makeDefaultMenu(){
    makeFileMenu(menuBar);
    makeEditMenu(menuBar);
    makeViewMenu(menuBar);
    makeToolsMenu(menuBar);
    makeHelpMenu(menuBar);
  }
  */
	
  /**
   * Makes a default menubar and toolbar.
   * Menus: File, Edit, View, Help
   */
  public void makeDefaultMenuToolbar(){
    makeFileMenuTool(menuBar);
    makeEditMenuTool(menuBar);
    makeViewMenuTool(menuBar);
    makeToolsMenuTool(menuBar);
    makeHelpMenuTool(menuBar);
  }
  /**
   * Makes a default file menu and toolbar buttons.
   * Menu items: new, open, save, save as, S, Options, S, close, exit
   * Toolbar buttons: new, open, save
   */
  public JMenu makeFileMenuTool(JComponent parentMenu) {
    //fileMenu = this.makeMenu(parentMenu,"File");
    makeFileMenuToolFiles(fileMenu);
    makeOptionsMenuTool(fileMenu);
    makeFileMenuToolExits(fileMenu);
    return fileMenu;
  }
  /**
   * Makes default file new/open/save/saveAs menu items and toolbar buttons.
   * Menu items: new, open, save, save as, S
   * Toolbar buttons: new, open, save
   */
  public void makeFileMenuToolFiles(JMenu fileMenu) {
    /*
    makeMenuTool(fileMenu,"new", newIcon, "New");//, 16
    makeMenuTool(fileMenu,"open","Open...", openIcon, "Open");//, 67
    makeMenuTool(fileMenu,"save", saveIcon, "Save");//, 118
    makeMenuItem(fileMenu,"saveAs","Save As...",'A');
    fileMenu.addSeparator();
    */
    newItem.show();
    openItem.show();
    saveItem.show();
    saveAsItem.show();
    fileMenu.addSeparator();
  }
  /**
   * Makes default close/exit menu options and and toolbar buttons.
   * Menu items: S, close, exit
   * Toolbar buttons: none
   */
  public void makeFileMenuToolExits(JMenu fileMenu) {
    /*
    fileMenu.addSeparator();
    makeMenuItem(fileMenu,"close","Close");
    makeMenuItem(fileMenu,"exit","Exit");
    */
    fileMenu.addSeparator();
    closeItem.show();
    exitItem.show();
  }
  /**
   * Makes a default options menu and toolbar buttons.
   * <nl><em>Menu items:</em> cut, copy, paste
   * <nl><em>Toolbar buttons:</em> cut, copy, paste
   */
  public JMenu makeOptionsMenuTool(JComponent parentMenu) {
    optionsMenu = makeMenu(fileMenu,"Options");
    JCheckBoxMenuItem cmi = makeCheckMenuItem(optionsMenu,"toolbar","Toolbar");
    cmi.setState(true);
    return optionsMenu;
  }
  /**
   * Makes a default edit menu and toolbar buttons.
   * <nl><em>Menu items:</em> cut, copy, paste
   * <nl><em>Toolbar buttons:</em> cut, copy, paste
   */
  public JMenu makeEditMenuTool(JComponent parentMenu) {
    //editMenu = this.makeMenu(parentMenu,"Edit");
    toolBar.addSeparator();
    /*
    makeMenuTool(editMenu, "cut", 'X', cutIcon, 
		 "Cut the selection and put it on the Clipboard");
    makeMenuTool(editMenu, "copy", 'C', copyIcon,
		 "Copy the selection and put it on the Clipboard");
    makeMenuTool(editMenu, "paste", 'V', pasteIcon, 
		 "Insert Clipboard contents");
    return editMenu;
    */
    
    cutItem.show();
    copyItem.show();
    pasteItem.show();
    return editMenu;
  }
  /**
   * Makes a default view menu and toolbar buttons.
   * <nl><em>Menu items:</em> none
   * <nl><em>Toolbar buttons:</em> none
   */
  public JMenu makeViewMenuTool(JComponent parentMenu) {
    //viewMenu = makeMenu(parentMenu,"View");
    return viewMenu;
  }
  /**
   * Makes a default tools menu and toolbar buttons.
   * <nl><em>Menu items:</em> none
   * <nl><em>Toolbar buttons:</em> none
   */
  public JMenu makeToolsMenuTool(JComponent parentMenu) {
    //toolsMenu = makeMenu(parentMenu,"Tools");
    return toolsMenu;
  }
  /**
   * Makes a default help menu and toolbar buttons.
   * <nl><em>Menu items:</em> help, manual, about
   * <nl><em>Toolbar buttons:</em> help
   */
  public JMenu makeHelpMenuTool(JComponent parentMenu) {
    //helpMenu = this.makeMenu(parentMenu,"Help");
    toolBar.addSeparator();
    /*
    makeMenuTool(helpMenu,"help", helpIcon, "Help"); 
    makeMenuItem(helpMenu,"manual", "Manual...", 'M');
    makeMenuItem(helpMenu,"about", "About...", 'B');
    return helpMenu;
    */
    helpItem.show();
    manualItem.show();
    aboutItem.show();
    return helpMenu;
  }


  /**
   * Makes a menu item and a toolbar button with command, icon, and
   * ToolTipText.
   * Uses the command to work out the label (capitalised command)
   * 
   * @param menuParent the menu that the item should be added to
   * @param command attached to item and button for recognising actions
   * @param icon the icon to use on the button
   * @param tttxt the tool tip text attached to the button
   * @return the new menu item
   */
  public JMenuItem makeMenuTool(JComponent menuParent, String command, 
			     Icon icon, String tttxt){
    char upFirst = Character.toUpperCase(command.charAt(0));
    String label = upFirst + command.substring(1);
    this.makeToolbarButton(command, icon, tttxt);
    return this.makeMenuItem(menuParent, command, label, icon);
  }
  /**
   * Makes a menu item and a toolbar button as above but with given label. 
   * Useful for items like "Open..."
   * 
   * @param menuParent the menu that the item should be added to
   * @param command attached to item and button for recognising actions
   * @param label the label to display on the menu item (usually ...)
   * @param icon the icon to use on the button
   * @param tttxt the tool tip text attached to the button
   * @return the new menu item
   */
  public JMenuItem makeMenuTool(JComponent menuParent, String command,
				String label, Icon icon, String tttxt){
    makeToolbarButton(command, icon, tttxt);
    return makeMenuItem(menuParent, command, label, icon);
  }
  /**
   * Makes a menu item and a toolbar button with given mnemonic. 
   * Same as the version above but with given mnemonic.
   * uses the command to work out the label (capitalised command) and the
   * mnemonic to work out the accelerator (mnemonic with CTRL_MASK)
   * 
   * @param menuParent the menu that the item should be added to
   * @param command attached to item and button for recognising actions
   * @param mnemonic used for mnemonic and accelerator (e.g. X for cut)
   * @param icon the icon to use on the button
   * @param tttxt the tool tip text attached to the button
   * @return the new menu item
   */
  public JMenuItem makeMenuTool(JComponent menuParent, String command,
				char mnemonic, Icon icon, String tttxt){
    char upFirst = Character.toUpperCase(command.charAt(0));
    String label = upFirst + command.substring(1);
    KeyStroke accel = KeyStroke.getKeyStroke(mnemonic,Event.CTRL_MASK);
    this.makeToolbarButton(command, icon, mnemonic, tttxt);
    return makeMenuItem(menuParent, command, label, mnemonic, icon, accel);
  }
  /**
   * Makes a menu item and a toolbar button as above but with given label. 
   * 
   * @param menuParent the menu that the item should be added to
   * @param command attached to item and button for recognising actions
   * @param label the label to display on the menu item (usually ...)
   * @param mnemonic used for mnemonic and accelerator (e.g. X for cut)
   * @param icon the icon to use on the button
   * @param tttxt the tool tip text attached to the button
   * @return the new menu item
   */
  public JMenuItem makeMenuTool(JComponent menuParent, String command,
				String label, char mnemonic, Icon icon, 
				String tttxt){
    KeyStroke accel = KeyStroke.getKeyStroke(mnemonic,Event.CTRL_MASK);
    this.makeToolbarButton(command, icon, mnemonic, tttxt);
    return makeMenuItem(menuParent, command, label, mnemonic, icon, accel);
  }


  /**
   * Makes a menu item and a toolbar button as with makeMenuTool but adds the
   * label to the icon too. Useful for items like "Modify draft"
   * 
   * @param menuParent the menu that the item should be added to
   * @param command attached to item and button for recognising actions
   * @param label the label to display on the menu item (usually ...)
   * @param icon the icon to use on the button
   * @param tttxt the tool tip text attached to the button
   * @return the new menu item
   */
  public JMenuItem makeMenuToolITxt(JComponent menuParent, String command,
				String label, Icon icon, String tttxt){
    JButton button = makeToolbarButton(command, icon, tttxt);
    button.setText(label);
    button.setVerticalTextPosition(AbstractButton.BOTTOM);
    button.setHorizontalTextPosition(AbstractButton.CENTER);
    return makeMenuItem(menuParent, command, label, icon);
  }


  /**
   * Makes a toolbar button with the given command, icon, and ToolTipText. 
   * The button is added to the toolbar of the frame and the frame is added
   * to the button's action listeners.
   * 
   * @param command attached to the button for recognising actions
   * @param icon the icon to use on the button
   * @param tttxt the tool tip text attached to the button
   */
  protected JButton makeToolbarButton(String command, Icon icon, String tttxt){
    JButton button = new JButton(icon);
    button.setDefaultCapable(false);
    button.setToolTipText(tttxt);
    button.setActionCommand(command);
    button.addActionListener(this);
    this.toolBar.add(button);
    return button;
  }
  /**
   * Makes a toolbar button as above but with a mnemonic. 
   * 
   * @param command attached to the button for recognising actions
   * @param mnemonic used for mnemonic and accelerator (e.g. X for cut)
   * @param icon the icon to use on the button
   * @param tttxt the tool tip text attached to the button
   */
  protected JButton makeToolbarButton(String command, Icon icon, 
				      char mnemo, String tttxt){
    JButton button = new JButton(icon);
    button.setDefaultCapable(false);
    button.setToolTipText(tttxt);
    button.setMnemonic((int)mnemo);
    button.setActionCommand(command);
    button.addActionListener(this);
    this.toolBar.add(button);
    return button;
  }
  /**
   * Makes a toolbar button with the given command, icon, text, and
   * ToolTipText. 
   * The button is added to the toolbar of the frame and the frame is added
   * to the button's action listeners.
   * 
   * @param command attached to the button for recognising actions
   * @param icon the icon to use on the button
   * @param tttxt the tool tip text attached to the button
   */
  protected JButton makeToolbarButton(String command, Icon icon, String text,
				      String tttxt){
    JButton button = new JButton(text, icon);
    button.setDefaultCapable(false);
    button.setToolTipText(tttxt);
    button.setActionCommand(command);
    button.addActionListener(this);
    this.toolBar.add(button);
    return button;
  }
  /**
   * Makes a toolbar button as above but with a given location.
   * The button is added at the given x-location in the toolbar of the frame
   * 
   * @param command attached to the button for recognising actions
   * @param mnemonic used for mnemonic and accelerator (e.g. X for cut)
   * @param icon the icon to use on the button
   * @param x the x-location for the button on the toolbar -***needed?
   * @param tttxt the tool tip text attached to the button
   */
  protected JButton makeToolbarButton(String command, Icon icon, int x, 
				      char mnemo, String tttxt){
    JButton button = new JButton(icon);
    button.setDefaultCapable(false);
    button.setToolTipText(tttxt);
    button.setMnemonic((int)mnemo);
    button.setActionCommand(command);
    button.addActionListener(this);
    this.toolBar.add(button);
    placeToolbarButton(button, x);
    return button;
  }

  public void placeToolbarButton(JButton button, int x) {
    button.setBounds(x, this.TBB_Y, this.TBB_WIDTH, this.TBB_HEIGHT);
  }


  /**
   * Makes a menu item with the given label and command and adds it to the
   * given (menu) parent. 
   *
   * @param menuParent the menu that the item should be added to
   * @param command attached to the item for recognising actions
   * @param label displayed in the menu
   */
  public JMenuItem makeMenuItem(JComponent parent, String command, 
				   String label){
    JMenuItem item = new javax.swing.JMenuItem(label);	 
    item.setActionCommand(command);
    item.addActionListener(this);
    parent.add(item);
    return item;
  }
  /**
   * Makes a menu item as above but with a given mnemonic. 
   * @param menuParent the menu that the item should be added to
   * @param command attached to the item for recognising actions
   * @param label displayed in the menu
   * @param mnemonic used for mnemonic and accelerator (e.g. X for cut)
   * as above but without the accelerator or an icon
   */
  public JMenuItem makeMenuItem(JComponent parent, String command, 
				   String label, char mnemonic){
    JMenuItem item = this.makeMenuItem(parent, command, label);
    item.setMnemonic((int)mnemonic);
    return item;
  }
  /**
   * Makes a menu item as above but with a given icon and accelerator
   * @param menuParent the menu that the item should be added to
   * @param command attached to the item for recognising actions
   * @param label displayed in the menu
   * @param mnemonic used for mnemonic and accelerator (e.g. X for cut)
   * @param icon the icon to be attached to the item
   * @param accel the key accelerator to be used for the item
   */
  public JMenuItem makeMenuItem(JComponent parent, String command, 
				   String label, Icon icon){
    JMenuItem item = this.makeMenuItem(parent, command, label);
    item.setIcon(icon);
    return item;
  }
  /**
   * Makes a menu item as above but with a given icon and accelerator
   * @param menuParent the menu that the item should be added to
   * @param command attached to the item for recognising actions
   * @param label displayed in the menu
   * @param mnemonic used for mnemonic and accelerator (e.g. X for cut)
   * @param icon the icon to be attached to the item
   * @param accel the key accelerator to be used for the item
   */
  public JMenuItem makeMenuItem(JComponent parent, String command, 
				   String label, char mnemonic,
				   Icon icon, KeyStroke accel){
    JMenuItem item = this.makeMenuItem(parent, command, label, mnemonic);
    item.setAccelerator(accel);
    item.setIcon(icon);
    return item;
  }

  /**
   * Makes a check box menu item and adds it to the given (menu) component.
   * As above but without the accelerator or the mnemonic
   *
   * @param menuParent the menu that the item should be added to
   * @param command attached to the item for recognising actions
   * @param label displayed in the menu
   */
  public JCheckBoxMenuItem makeCheckMenuItem(JComponent parent, 
						String command, String label){
    JCheckBoxMenuItem item = new javax.swing.JCheckBoxMenuItem(label);	 
    item.setActionCommand(command);
    item.addActionListener(this);
    parent.add(item);
    return item;
  }

  /**
   * Makes a radio button menu item and adds it to the given (menu) component.
   *
   * @param menuParent the menu that the item should be added to
   * @param command attached to the item for recognising actions
   * @param label displayed in the menu
   */
  public JRadioButtonMenuItem makeRadioMenuItem(JComponent parent, 
						String command, String label){
    JRadioButtonMenuItem item = new JRadioButtonMenuItem(label);	 
    item.setActionCommand(command);
    item.addActionListener(this);
    parent.add(item);
    return item;
  }

  /**
   * Makes a menu and adds it to the given (menu) component.
   * Sets the accelerator to be the first letter of the label
   *
   * @param menuParent the menu that the item should be added to
   * @param label the name of the menu
   */
  public JMenu makeMenu(JComponent parent, String label){
    JMenu menu = new javax.swing.JMenu(label);	 
    menu.setActionCommand(label);
    menu.setMnemonic((int)label.charAt(0));
    parent.add(menu);
    return menu;
  }

  public void actionPerformed(java.awt.event.ActionEvent event) {
      Object object = event.getSource();
      String command = ((AbstractButton)object).getActionCommand();
      //Debug.noteln("action object is",object);
      Debug.noteln("JFrameMT: action command is",command);
      if (command == "open")	this.openFile();
      else if (command == "save") this.saveFile();
      else if (command == "toolbar") 
	this.toggleToolbar(((JCheckBoxMenuItem)object).getState());
      else if (command == "close") this.setVisible(false);
      else if (command == "exit") this.exit();
      else if (command == "about") this.aboutDialog();
      else if (command == "open") this.openFile();
      else if (command == "saveAs") this.saveFileAs(event);
      //else if (command == "about") this.aboutDialog();
      else {
	String txt = ((AbstractButton)object).getText();
	if ((txt == "") || txt == null) txt = command;
	UIUtil.notImplemented(this, txt);
      }
  }

  public void exitApplication() {
    try {
      // Beep, then show a confirmation dialog
      Toolkit.getDefaultToolkit().beep();
      int reply = JOptionPane.showConfirmDialog(this, 
						"Do you really want to exit?", 
						"JFC Application - Exit" , 
						JOptionPane.YES_NO_OPTION, 
						JOptionPane.QUESTION_MESSAGE);
      // If the confirmation was affirmative, handle exiting.
      if (reply == JOptionPane.YES_OPTION){
	this.setVisible(false);    // hide the Frame
	this.dispose();            // free the system resources
	System.exit(0);            // close the application
      }
    } catch (Exception e) {}
  }

  public void openFile() {
    try {
      openFileDialog.setVisible(true);
    } catch (Exception e) {}
  }
  
  public void saveFile()
  {
    UIUtil.notImplemented(this,"Saving files");
  }
  
  public void saveFileAs(java.awt.event.ActionEvent event)
  {
    // to do: code goes here.
    try {saveFileDialog.setVisible(true);  // Show the FileDialog
    } catch (Exception e) {}
  }
  
  public void toggleToolbar(boolean on){
    Rectangle bounds = mainJPanel.getBounds();
    int y,height,tbHeight;
    tbHeight = (int)this.JPanel2.getHeight();
    if (on) {
      y = tbHeight; 
      height = (int)bounds.getHeight() - tbHeight;}
    else {y = 0; height = (int)bounds.getHeight() + tbHeight;}
    JPanel2.setVisible(on);
    mainJPanel.setBounds(0,y,(int)bounds.getWidth(),height);
    mainJPanel.doLayout();
    //getContentPane().doLayout();
    //this.setVisible(false);
    //this.doLayout();
    //this.setVisible(true);
    this.validate();
  }
  
  public void exit(){
    // to do: code goes here.
    try {this.exitApplication();} catch (Exception e) {}
  }
  
  public void aboutDialog(){
    try { 
      ensureAboutDialog();
      aboutDialog.show();
    } catch (Exception e) {}
  }
  // JAboutDialog Create with owner and show as modal
  private void ensureAboutDialog(){
    if (aboutDialog == null) {
      aboutDialog = new JAboutDialog(this);
      aboutDialog.setModal(true);
    }
  }
}


/**
 * An About dialog that is to be adapted for the application.
 *
 * This was generated automatically bu VisualCafe and is now waiting 
 * to be incorporated properly.
 */
class JAboutDialog extends javax.swing.JDialog {
  protected Frame frame;

  public JAboutDialog(JFrame parentFrame) {
    super(parentFrame);
    this.frame = parentFrame;
    // This code is automatically generated by Visual Cafe when you add
    // components to the visual environment. It instantiates and initializes
    // the components. To modify the code, only use code syntax that matches
    // what Visual Cafe can generate, or Visual Cafe may be unable to back
    // parse your Java file into its visual environment.
    //{{INIT_CONTROLS
    this.setTitle(this.frame.getTitle() + " - About");
    setModal(true);
    getContentPane().setLayout(new GridBagLayout());
    setSize(248,94);
    setVisible(false);
    okButton.setText("OK");
    okButton.setActionCommand("OK");
    okButton.setOpaque(false);
    okButton.setMnemonic((int)'O');
    GridBagConstraints gbc;
    Insets insets = new Insets(0,0,10,0);
    gbc = new GridBagConstraints(2,1,1,1,0.0,0.0,
	GridBagConstraints.CENTER,GridBagConstraints.NONE,insets,0,0);
    getContentPane().add(okButton, gbc);
    okButton.setBounds(98,59,51,25);
    aboutLabel.setHorizontalAlignment(SwingConstants.CENTER);
    aboutLabel.setText(this.frame.getTitle());
    insets = new Insets(0,0,0,0);
    gbc = new GridBagConstraints(0,0,3,1,1.0,1.0,
	GridBagConstraints.CENTER,GridBagConstraints.BOTH,insets,0,0);
    getContentPane().add(aboutLabel, gbc);
    aboutLabel.setBounds(0,0,248,59);
    //}}

    //{{REGISTER_LISTENERS
    AboutWindow aAboutWindow = new AboutWindow();
    this.addWindowListener(aAboutWindow);
    AboutAction aAboutAction = new AboutAction();
    okButton.addActionListener(aAboutAction);
    //}}
  }

  public void setVisible(boolean b) {
    if (b){
      Rectangle bounds = (getParent()).getBounds();
      Dimension size = getSize();
      setLocation(bounds.x + (bounds.width - size.width)/2,
		  bounds.y + (bounds.height - size.height)/2);
    }
    super.setVisible(b);
  }

  public void addNotify()
  {
    // Record the size of the window prior to calling parents addNotify.
    Dimension d = getSize();
    
    super.addNotify();
    
    if (fComponentsAdjusted) return;
    // Adjust components according to the insets
    Insets insets = getInsets();
    setSize(insets.left + insets.right + d.width, 
	    insets.top + insets.bottom + d.height);
    Component components[] = getContentPane().getComponents();
    for (int i = 0; i < components.length; i++) {
      Point p = components[i].getLocation();
      p.translate(insets.left, insets.top);
      components[i].setLocation(p);
    }
    fComponentsAdjusted = true;
  }

  // Used for addNotify check.
  boolean fComponentsAdjusted = false;
  
  //{{DECLARE_CONTROLS
  javax.swing.JButton okButton = new javax.swing.JButton();
  javax.swing.JLabel aboutLabel = new javax.swing.JLabel();
  //}}
  
  class AboutWindow extends java.awt.event.WindowAdapter
  {
    public void windowClosing(java.awt.event.WindowEvent event)
    {
      Object object = event.getSource();
      if (object == JAboutDialog.this)
	JAboutDialog.this.closeAbout();
    }
  }
  
  class AboutAction implements java.awt.event.ActionListener {
    public void actionPerformed(java.awt.event.ActionEvent event)
    {
      Object object = event.getSource();
      if (object == okButton) JAboutDialog.this.closeAbout();
    }
  }
  void closeAbout(){
    try {this.setVisible(false);// JAboutDialog Hide the JAboutDialog
    } catch (Exception e) {}				
  }
  
}
/*
 * Document all public methods
 */
