/****************************************************************************
 * A simple panel with a label, a text component, and optional buttons.
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iface.ui;

import javax.swing.*;
import javax.swing.text.JTextComponent;
import java.awt.event.*;

import ix.util.Debug;
    
/**
 * A simple panel with a label (title), a component, and optional buttons.
 * The given label will be at the top, the given component at the left and
 * buttons with given labels on the right.
 * Adds the given listener to the buttons and the list.
 */
public class ThingEditorPanel extends AbstractEditorPanel {

        
  public ThingEditorPanel(){super();}
  public ThingEditorPanel(MouseListener ml, String label, 
			  JComponent component) { 
    super(ml, label, component);
  }
  public ThingEditorPanel(MouseListener ml, String label, String[] buttons) { 
    super(ml, label, buttons);
  }
  public ThingEditorPanel(MouseListener ml, String label, 
			  JComponent component, String[] buttons) { 
    super(ml, label, component, buttons);
  }
  public ThingEditorPanel(MouseListener ml, boolean bordered, String label, 
			  JComponent component) { 
    super(ml, bordered, label, component);
  }

  public boolean isListItem() { return false;}

  /**
   * Gets the current data of a text component.
   * @return data a string that contains the text from component.
   */
  public Object getData() {
    return ((JTextComponent)this.itemComponent).getText().trim();
  }

  /**
   * Sets the current data of a text component.
   * @param data a string that contains the text to put into the component
   */
  public void setData(Object data) {
    //texts may sit in scrollPanes
    JTextComponent textComp;
    if (this.itemComponent.getClass() == JScrollPane.class) {
	JScrollPane pane = (JScrollPane) this.itemComponent;        
	textComp = (JTextComponent) pane.getViewport().getComponent(0);            
    } else { textComp = (JTextComponent) this.itemComponent;}
    textComp.setEnabled(false);
    if (data == null) textComp.setText("");    
    else textComp.setText(data.toString());    
    textComp.setEnabled(true);
  }

  /**
   * Removes all text from the component.
   */
  public void removeData() {setData("");}

  public boolean setRenderer(ListCellRenderer r) {return false;}

}
