/****************************************************************************
 * Abstract table that listens to the mouse, has values with up to 4D and
 * sensible column widths.
 * 
 * @author Jussi Stader
 * @version 1.0
 ****************************************************************************
 */
package ix.iface.ui.table;


import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import ix.iface.ui.*;
import ix.util.*;

/****************************************************************************
 * Abstract table that listens to the mouse, has values with up to 4D and
 * sensible column widths.
 *
 * The IXTable is used to provide a standard table behaviour for displaying
 * up to fourD displays of fields (as provided by the NDRenderer), adjusted
 * column widths, no re-ordering of columns, adding the model as a
 * TableModelListener if it is one.<p>
 * Can be used for normal tables or tree tables. <p>
 *
 * Example code for making a TreeTable happen:
 * <PRE><code>
 * ...
 *   TreeNode[] tna = {};
 *   TreeNode[] nodes = domain.getAllRefinements().toArray(tna);
 *   ActionTreeTableModel tableModel = new ActionTreeTableModel(domain, nodes);
 *   // or simply: tableModel = new ActionTreeTableModel(domain);
 *   IXTable table = new IXTreeTable(tableModel);
 *   table.setPreferredScrollableViewportSize(new Dimension(200, 100));
 *   JScrollPane jsp = new JScrollPane();
 *   jsp.getViewport().add(table);
 *   panel.add(jsp);
 * ...
 * </code></PRE>
 ****************************************************************************
 */

public class IXTreeTable extends IXTable {

  public IXTreeTable(){
    //    Insets ins = getInsets();
    //    setSize(ins.left + ins.right + 430,ins.top + ins.bottom + 270);
    super();
    setTreeTypeRenderer();
  }

  /** 
   * Makes a table and sets the given model, adjusts column widths, registers
   * model as mouse listener (if appropriate).
   */
  public IXTreeTable(AbstractTableModel model) {
    super(model);
    setTreeTypeRenderer();
  }

  private void setTreeTypeRenderer() {
    Debug.noteln("IXTT: Setting neutral icon renderer");
    JTableHeader header = getTableHeader();
    this.setDefaultRenderer(ImageIcon.class, 
			    new NeutralIconRenderer(false, header));
  }
}
