/* File: UIUtil.java
 * Contains: A class for useful static methods for interfaces
 * Author: Jussi Stader <J.Stader@ed.ac.uk>
 * Created: July 2001
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 */

package ix.iface.ui.util;

import ix.util.*;
import ix.util.lisp.*;
import java.lang.reflect.*;
import java.util.*;
import java.io.*;
import java.net.URL;
import javax.swing.*;
import java.awt.Component;
import java.awt.event.*;


/** Class for useful static methods. */

public class UIUtil {

    /**
     * Brings up a message that the given string item is not yet supported.
     * e.g. "Editing AROs is not yet supported"
     */
    public static void notImplemented(Component parent, String item){
      JOptionPane.showMessageDialog(parent, 
				    item + " is not yet supported.");
    }
    /**
     * Brings up a message that the given string item is not yet supported.
     * e.g. "Editing AROs is not yet supported"
     */
    public static void warning(Component parent, String item){
      JOptionPane.showMessageDialog(parent, "Warning: " + item);
    }


  //---------------------------Menu things

  /**
   * Fills the given menu with the names of the given objects, attaching the
   * given command.
   * Always clears the menu first and disables it. Only enables it if there are
   * items in it.
   * 
   * @param menu the menu to be filled
   * @param command attached to each new menu item
   * @param objects an ArrayList of named objects to be put in the menu
   */
  public static void populateMenu(ActionListener al, JMenu menu, 
				  String command, List objects) {
    menu.setEnabled(false);
    menu.removeAll();
    if ((objects != null) && (!objects.isEmpty())) {
      int max = 20;
      int remaining = objects.size();
      int count = 0;
      JMenu shortMenu = menu;
      for (Iterator i = objects.iterator(); i.hasNext();) {
	if (count == max && remaining > 5) {
	  JMenu submenu = new JMenu("More");
	  shortMenu.add(submenu);
	  shortMenu = submenu;
	  count = 0;
	}
	ix.icore.domain.Named o = (ix.icore.domain.Named)i.next();
	makeMenuItem(al, shortMenu, command, o.getName());
	remaining--;
	count++;
      }
      menu.setEnabled(true);
    }
  }

  /**
   * Makes a menu item with the given label and command and adds it to the
   * given (menu) parent. 
   *
   * @param al the action listener that is interested in the item's selection
   * @param menu the menu that the item should be added to
   * @param command attached to the item for recognising actions
   * @param label displayed in the menu
   */
  public static JMenuItem makeMenuItem(ActionListener al, JComponent menu, 
				       String command, String label){
    JMenuItem item = new javax.swing.JMenuItem(label);	 
    item.setActionCommand(command);
    item.addActionListener(al);
    menu.add(item);
    return item;
  }

  //---------------------------String things

  public static String toCapitalised(String inString) {
    char upFirst = Character.toUpperCase(inString.charAt(0));
    return upFirst + inString.substring(1);
  }
  public static String toUncapitalised(String inString) {
    char upFirst = Character.toLowerCase(inString.charAt(0));
    return upFirst + inString.substring(1);
  }

  public static class ToStringComparator implements Comparator {
    public int compare(Object o1, Object o2) {
      try {
	String s1 = o1.toString();
	String s2 = o2.toString();
	return s1.compareTo(s2);
      }
      catch (Exception cce) {
	Debug.noteException(cce);
	return 0;
      }
    }

    //public boolean equals(Object o) {
    //  return super.equals(o);
    //}
  }


    /**
     * breakStringAtFirst takes a string containing fields separated by
     * a (string) delimiter and returns a two-element string array containing
     * the substring before the first occurrence of the char, and the
     * substring after.  Neither substring contains the delimiter.  If
     * the delimiter does not appear in the string at all, the values
     * are the string and "".
     */
    public static String[] breakStringAtFirst(String s, String separator) {
	int i = s.indexOf(separator);
	if (i == -1)
	    return new String[]{s, ""};
	else
	    return new String[]{
		s.substring(0, i),
	        s.substring(i + separator.length())
	    };
    }


  //---------------------------Field things

  public static Field stringToField(Class c, String s) {
    try {
      Field field = c.getField(s);
      return(field);
    }
    catch (NoSuchFieldException e) {
      Debug.noteln("Cannot find field " + s + " in class " + c.getName());
      return null;
    }
  }

  public static Object getObjectFieldValue(Object object, String field) {
    if (object == null) {
      Debug.noteln("Util: Cannot get value for a null object.");
      return null;
    }
    else {
      try {
	Field oField = stringToField(object.getClass(), field);
	return oField.get(object);
      }
      catch (Exception e) {
	Debug.noteln("Cannot get field " + field + " Trying getField method");
	String mName = "get" + toCapitalised(field);
	try {
	  Class[] args = {};
	  Method method = object.getClass().getMethod(mName, args);
	  return method.invoke(object, args);
	}
	catch (Exception me) {
	  Debug.noteException(me);
	  return null;
	}  
      }
    }
  }
  public static void setObjectFieldValue(Object object, String field,
					 Class fieldClass, Object value) {
    if (object == null) {
      Debug.noteln("Util: Cannot set value for a null object.");
      return;
    }
    if (field == null) {
      Debug.noteln("Util: Cannot set value for a null field.");
      return;
    }
    else {
      try {
	Field oField = stringToField(object.getClass(), field);
	oField.set(object, value);
      }
      catch (Exception e) {
	Debug.noteln("Cannot set field " + field + " Trying setField method");
	String mName = "set" + toCapitalised(field);
	try {
	  Class valClass;
	  //if (value == null) valClass = Object.class;
	  //else valClass = value.getClass();
	  Class[] classes = {fieldClass};
	  Method method = object.getClass().getMethod(mName, classes);
	  if (value == null) value = fieldClass.newInstance();
	  else Debug.noteln("Value is  ", value.toString());
	  Object[] args = {value};
	  method.invoke(object, args);
	}
	catch (Exception me) {
	  Debug.noteln("setObjectFieldValue: problem with field " + field);
	  Debug.note(" method " + mName);
	  Debug.noteln(", value class" + value.getClass().getName());
	  Debug.noteException(me);
	  return;
	}  
      }
    }
  }


  //---------------------------List things

  public static String show(Collection l) {
    String text = "";
    Iterator i = l.iterator();
    while (i.hasNext()) 
      text = text + i.next().toString() + " ";
    return text;
  }
  public static String listToDisplay(Collection l) {
    String text = "";
    Iterator i = l.iterator();
    while (i.hasNext()) 
      text = text + i.next().toString() + " ";
    return text;
  }
  public static String fieldsToNames(Field[] fields) {
    String text = "";
    for (int i=0; i<fields.length; i++) 
      text = text + fields[i].getName() + " ";
    return text;
  }


  public static LList removeDuplicates(LList list) {
    if ((list == null) || (list.isEmpty()) || (list.isNull())) return list;
    else {
      Object o = list.car();
      LList withoutCdr = remove(o, list.cdr());
      LList newCdr = removeDuplicates(withoutCdr);
      return new Cons(o, newCdr);
    }
  }

  public static LList remove(Object o, LList list) {
    if ((list == null) || (list.isEmpty()) || (list.isNull())) return list;
    else {
      Object car = list.car();
      if (o.equals(car)) return remove(o, list.cdr());
      else return new Cons(car, remove(o, list.cdr()));
    }
  }



  //--------------------------- things

    public static URL resourceURL(String filename) {
        String fullname = "resources/html/" + filename;
        URL result = UIUtil.class.getClassLoader().getResource(fullname);
        Debug.noteln("Making resource URL", result);
        return result;
	/*
	String base = System.getProperty("ix.base", ".");
	File resources = new File(base, "resources");
	File html = new File(resources, "html");
	File urlLocation = new File(html, filename);
	Debug.noteln("Making URL from", urlLocation);
	try {
	    return urlLocation.toURL();
	}
	catch (java.net.MalformedURLException e) {
	    Debug.noteException(e);
	    throw new RuntimeException("Can't handle URL filename "
				       + filename);
	}
	*/
    }

  //--------------------------- things


  //--------------------------- things



}
