/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Sun Jun  9 19:03:34 2002 by Jeff Dalton
 * Copyright: (c) 2002, AIAI, University of Edinburgh
 */

package ix.ip2;

import java.util.*;

import ix.icore.*;

import ix.ispace.*;

/**
 * Generates HandlerActions for an AgendItem.
 */
public class InvokeHandler extends ItemHandler {

    protected IXAgent agent;

    public InvokeHandler(IXAgent agent) {
	super("invoke");	// action description
	this.agent = agent;
    }

    // /\/: Will eventually need to add new actions when an
    // agent gains a capability or a new agent with the capability
    // is added.

    public void addHandlerActions(AgendaItem item) {
	// It's assumed the controller has already determined that
	// this is an appropriate handler for the item.
	String verb = item.getAbout().getVerb().toString();
	Capability required = new VerbCapability(verb);
	List agents = agent.getContactManager()
	    .getAgentData(required);
	for (Iterator i = agents.iterator(); i.hasNext();) {
	    AgentData data = (AgentData)i.next();
	    item.addAction(new InvokeAction(data.getName(), required));
	}
    }

    public void handle(AgendaItem item) {
	throw new Error(this + " called directly");
    }

    class InvokeAction extends HandlerAction {

	String toName;
	Capability required;

	InvokeAction(String toName, Capability required) {
	    this.toName = toName;
	    this.required = required;
	    this.shortDescription = "invoke " + toName;
	}
	
	public void handle(AgendaItem item) {
	    item.getAbout().forwardTo(toName, true);
	    item.setStatus(Status.EXECUTING);
        }

	public boolean isStillValid() {
	    // This action should be removed if the agent is
	    // deleted or loses the capability.
	    AgentData data = agent.getContactManager().getAgentData(toName);
	    return data != null
		&& data.hasCapability(required);
	}

    }

}
