/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Thu May 23 00:03:11 2002 by Jeff Dalton
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 */

package ix.ip2;

import javax.swing.*;

import java.awt.Toolkit;
import java.awt.Component;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.*;

import ix.ip2.*;

import ix.ichat.ChatFrame;
import ix.ispace.IspaceTool;

import ix.icore.*;
import ix.iview.*;
import ix.iface.util.LogoPanel;
import ix.iface.util.IconImage;
import ix.iface.util.CatchingActionListener;
import ix.util.*;

/**
 * The main frame of the I-P2 user interface.
 */
public class Ip2Frame extends PanelFrame implements ActionListener {

    Container contentPane;

    ChatFrame chatFrame;	// rename to messenger? /\/

    IspaceTool ispaceTool;

    // /\/: Do we need these fields?
    // /\/: Should it be productViewer or stateViewer?
    AgendaViewer activityViewer;
    AgendaViewer issueViewer;
    StateViewer stateViewer;

    Ip2 ip2;

    // /\/: May want some way to find menus my looking at the menu bar
    // components, rather than storing the menus as fields in the frame.
    JMenu fileMenu  = new JMenu("File");
    JMenu newMenu = new JMenu("New");
    JMenu toolsMenu = new JMenu("Tools");
    JMenu testMenu  = new JMenu("Test", true); // tear-off "Test" menu

    public Ip2Frame(Ip2 ip2) {
	super(ip2.getAgentDisplayName());
	this.ip2 = ip2;
    }

    protected void setUp() {
	setIconImage(IconImage.getIconImage(this));
	activityViewer = ip2.activityViewer;
	issueViewer = ip2.issueViewer;
	stateViewer = ip2.stateViewer;

	contentPane = getContentPane();

	setSize(500, 400);	// preliminary
	setJMenuBar(makeMenuBar());
	// Layout manager defaults to BorderLayout.
	// contentPane.setLayout(new FlowLayout()); // probably not too clever

	// Package issue viewer
	JPanel issuePanel = new JPanel();
	issuePanel.add((Component)issueViewer);
	JScrollPane issueScroll = new JScrollPane(issuePanel);
	issueScroll.setBorder(BorderFactory.createTitledBorder("Issues"));

	// Package activity viewer
	JPanel activityPanel = new JPanel();
	activityPanel.add((Component)activityViewer);
	JScrollPane activityScroll = new JScrollPane(activityPanel);
	activityScroll
	    .setBorder(BorderFactory.createTitledBorder("Activities"));

	// Package state viewer
	// stateViewer.setBorder(BorderFactory.createTitledBorder("inner"));
	JPanel statePanel = new JPanel();
	statePanel.add((Component)stateViewer);
	JScrollPane stateScroll = new JScrollPane(statePanel);
	stateScroll.setBorder(BorderFactory.createTitledBorder("State"));

	// Put the viewers in nested split panes.

	// JSplitPane split1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	// JSplitPane split2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	split1.setTopComponent(issueScroll);
	split1.setBottomComponent(split2);
	split2.setTopComponent(activityScroll);
	split2.setBottomComponent(stateScroll);
	// split.setDividerLocation(0.80); // the one after pack() matters /\/
	contentPane.add(split1);

	// Add something that identifies the system etc.
	contentPane.add(ip2.makeLogoPanel(),
			BorderLayout.SOUTH);

    }

    // Needs to be accessible in becomeVisible().
    JSplitPane split1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    JSplitPane split2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

    protected void becomeVisible() { // sep method so can do it last
	// It is expected that this is called only once.

	// pack() is mostly to get a good width, so remember the
	// original height because we'll probably want to keep it.
	Dimension origSize = getSize();
	int origHeight = (int)Math.round(origSize.getHeight());

	pack();

	// Keep the frame from being too tall.
	// The way to do it is somewhat empirical ... /\/
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension size = getSize();
	int width = (int)Math.round(size.getWidth());
	int height = (int)Math.round(size.getHeight());
	int maxHeight = (int)((7.0 / 8.0) * screenSize.getHeight());
	int scrollWidth = 30;	// scrollbar will appear /\/
	int desiredHeight = Math.max(Math.min(height, maxHeight),
				     origHeight);
	if (height != desiredHeight)
	    setSize(new Dimension(width + scrollWidth, desiredHeight));

	// Seem to need this here, rather than, say, later.
	super.validate();

	// Adjust the split pane dividers.
	// /\/: If we nest split1 in split2 instead of split2 in split1,
	// the divider locations are bizarre.
	split1.setDividerLocation(0.33);
	split2.setDividerLocation(0.55);

	// Finally, make it visible
	setVisible(true);
    }

    protected void ensureDomainEditor() {
	if (domainEditor == null) {
	    domainEditor = makeDomainEditor(ip2, ip2.domain);
	    // domainEditor.setLocation(getX() + 100, getY() + 100);
	}
	domainEditor.setVisible(true);
    }

    public void ensureChat() {
	ensureChatExists();
	chatFrame.setVisible(true);
    }

    public void ensureChatExists() {
	if (chatFrame == null) {
	    chatFrame = new ChatFrame
		(ip2, ip2.getAgentDisplayName() + " Messenger");
	}
    }

    public void ensureIspaceTool() {
	if (ispaceTool == null) {
	    ispaceTool = new IspaceTool(ip2);
	}
	ispaceTool.setVisible(true);
    }


    /*
     * Menu bar
     */

    protected JMenuBar makeMenuBar() {
	JMenuBar bar = new JMenuBar();

	// File menu
	bar.add(fileMenu);
	fileMenu.add(makeMenuItem("Reset"));
	fileMenu.addSeparator();
	fileMenu.add(makeMenuItem("Exit"));

	// Issue menu
	bar.add(newMenu);
	newMenu.add(makeMenuItem("New Issue"));
	newMenu.add(makeMenuItem("New Activity"));

	// Tools menu
	bar.add(toolsMenu);
	toolsMenu.add(makeMenuItem("Domain Editor"));
	toolsMenu.add(makeMenuItem("I-Space"));
	toolsMenu.add(makeMenuItem("Messenger"));

	bar.add(Box.createHorizontalGlue()); // let later items move right

	// Test menu
	bar.add(testMenu);
	testMenu.setEnabled(false); 	// initially
	ip2.addTestMenuItems();

	return bar;
    }

    protected JMenuItem makeMenuItem(String text) {
	JMenuItem item = new JMenuItem(text);
	item.addActionListener(CatchingActionListener.listener(this));
	return item;
    }

    /*
     * Action interpreter
     */

    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	Debug.noteln("I-P2 action:", command);
	if (command.equals("Exit")) {
	    if (Util.dialogConfirms(this, "Are you sure you want to exit?"))
		System.exit(0);		// should call agent exit /\/
	}
	else if (command.equals("Reset")) {
	    ip2.reset();
	}
	else if (command.equals("New Issue")) {
	    issueViewer.getNewItemFromUser();
	}
	else if (command.equals("New Activity")) {
	    activityViewer.getNewItemFromUser();
	}
	else if (command.equals("Domain Editor")) {
	    ensureDomainEditor();
	}
	else if (command.equals("Messenger")) {
	    ensureChat();
	}
	else if (command.equals("I-Space")) {
	    ensureIspaceTool();
	}
	else
	    Debug.noteln("Nothing to do for", command);
    }

//  /**
//   * Adds a menu item to the "File" menu.
//   */
//  public void addFileMenuItem(String text, ActionListener listener) {
//  	JMenuItem item = new JMenuItem(text);
//  	item.addActionListener(listener);
//  	// Add it before the separator before "Exit".
//  	fileMenu.insert(item, fileMenu.getItemCount() - 2);
//  }

    /**
     * Adds a test by adding a menu item to the "Test" menu.
     * The action listener will be wrapped in a CatchingActionListener.
     * All test additions should go directly or indirectly
     * through this method.
     *
     * @see ix.iface.util.CatchingActionListener
     */
    public void addTest(String text, ActionListener listener) {
	JMenuItem item = new JMenuItem(text);
	item.addActionListener(CatchingActionListener.listener(listener));
	testMenu.setEnabled(true);
	testMenu.add(item);
    }

    /**
     * Adds a test menu item based on a TestItem.
     */
    public void addTest(TestItem item) {
	final Sendable contents = item.getContents();
	final String toName = item.getToName();
	String menuText = Strings.replace("$to", toName, item.getMenuText());

	contents.setSenderId(Name.valueOf((String)ip2.getAgentIPCName()));

	addTest(menuText, new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		// N.B. must make a copy, because it will change
		// its status, etc.
		ensureChatExists();
		chatFrame.sendCopy(toName, contents);
	    }
	});

    }

}

// Issues:
// * Perhaps a more abstract name, such as PanelUserInterface, instead of
//   Ip2Frame.
