/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Thu May 30 18:01:45 2002 by Jeff Dalton
 * Copyright: (c) 2001 - 2002, AIAI, University of Edinburgh
 */

package ix.ip2;

import java.util.*;

import ix.icore.*;
import ix.util.*;
import ix.util.lisp.*;

/**
 * An agenda of issues
 */
public class IssueAgenda extends Agenda {

    AgendaItem openOtherReportsIssue = null; // if exists and
                                             // status not complete

    public IssueAgenda(PanelController controller) {
	super(controller);
    }

    public AgendaItem makeItem(LList pattern) {
	return new IssueItem(new Issue(pattern));
    }

    public void addItem(AgendaItem item) {
	item.setPattern(modelManager.putVariablesInPattern(item.getPattern()));
	item.computeStatus();
	super.addItem(item);
    }

    public boolean acceptReport(Report report) {
	// Pass 1 - skipping any OtherReportsIssue.
	for (Iterator i = items.iterator(); i.hasNext(); ) {
	    AgendaItem item = (AgendaItem)i.next();
	    if (item instanceof OtherReportsIssue)
		continue;
	    if (item.wantsReport(report)) {
		item.addReport(report);
		return true;
	    }
	}
	// Else use an OtherReportsIssue.
	if (openOtherReportsIssue == null) {
	    openOtherReportsIssue =
		new OtherReportsIssue("Note other reports");
	    addItem(openOtherReportsIssue);
	}
	openOtherReportsIssue.addReport(report);
	return true;
    }

    public class OtherReportsIssue extends IssueItem {

	public OtherReportsIssue(String text) {
	    super(new Issue(text));
	    setStatus(Status.EXECUTING);
	}

	boolean wantsReport(Report report) {
	    Debug.noteln("Other reports issue asked if it wants", report);
	    return getStatus() != Status.COMPLETE;
	}

	public boolean wantsActionsFrom(ItemHandler handler) {
	    // /\/: This is wrong but will work for now, because
	    // the actions we want all come directly from the agenda
	    // rather than from handlers.
	    return false;
	}

	boolean actionCanBeTakenNow(HandlerAction act) {
	    return getStatus() == Status.POSSIBLE 	//  ? /\/
		|| (act instanceof HandlerAction.Manual
		    && getStatus() == Status.EXECUTING);
	}

	public void setStatus(Status status) {
	   if (status == Status.COMPLETE
	       && this == IssueAgenda.this.openOtherReportsIssue)
	       IssueAgenda.this.openOtherReportsIssue = null;
	   super.setStatus(status);
	}

	void setStatusBasedOn(Report report) {
	    // We don't want to become COMPLETE or IMPOSSIBLE
	    // because of a report, so we override the inherited
	    // method.
	}

    }

}
