/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Tue May 28 21:53:53 2002 by Jeff Dalton
 * Copyright: (c) 2001 - 2002, AIAI, University of Edinburgh
 */

package ix.ip2;

import ix.icore.*;
import ix.util.*;

/**
 * Generates HandlerActions for an AgendaItem. <p>
 *
 * N.B. Handlers that can handle both issues and sctivities
 * extend this class rather than IssueHandler or ActivityHandler. 
 */
public class ItemHandler {

    protected String actionDescription;

    public ItemHandler(String actionDescription) {
	this.actionDescription = actionDescription;
    }

    public String getActionDescription() {
	return actionDescription;
    }

    /**
     * Gives this handler its say in whether it should add
     * actions to an item.
     *
     * @see AgendaItem#wantsActionsFrom(ItemHandler handler)
     */
    public boolean appliesTo(AgendaItem item) {
	// This method allows the handler to reject some items.
	// This is especially useful if the handler is added as
	// a "universal" handler.
	return true;
    }

    public void addHandlerActions(AgendaItem item) {
	// It's assumed the controller has already determined that
	// this is an appropriate handler for the item.
  	item.addAction
  	    (new HandlerAction.Automatic(this));
    }

    public void reviseHandlerActions(AgendaItem item, Object reason) {
	throw new UnsupportedOperationException
	    (this + "cannot revise handler actions");
    }

    public void handle(AgendaItem item) {
	Debug.noteln(this + " handling " + item);
    }

    public String toString() {
	return "ItemHandler[" + getActionDescription() + "]";
    }

}
