/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Thu May 30 23:13:44 2002 by Jeff Dalton
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 */

package ix.ip2;

import javax.swing.*;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.lang.reflect.Constructor;

import ix.icore.IXAgent;
import ix.icore.domain.Domain;
import ix.iview.*;

import ix.util.*;

/**
 * The main frame of a process panel user interface.
 */
public abstract class PanelFrame extends JFrame {

    protected InternalDomainEditor domainEditor;

    public PanelFrame(String title) {
	super(title);

	// Make sure a "close" doesn't just leave it running in the
	// background...
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		Debug.noteln("Main frame windowClosing event");
		if (Util.dialogConfirms(PanelFrame.this,
		       "Are you sure you want to exit?"))
		    System.exit(0);		// should call agent exit /\/
	    }
	});

    }

    public InternalDomainEditor makeDomainEditor(IXAgent agent, Domain dom) {
	// New editor: ix.iview.DomainEditor
	// Old editor: ix.iview.SimpleDomainEditor
	String className =
	    Parameters.getParameter
	        ("domain-editor-class",
		 // "ix.iview.DomainEditor"
		 "ix.iview.SimpleDomainEditor");
	try {
	    Debug.noteln("Trying domain editor class", className);
	    Class c = Class.forName(className);
	    Class[] consClasses = {IXAgent.class, Domain.class};
	    Constructor cons = c.getConstructor(consClasses);
	    InternalDomainEditor ed =
		(InternalDomainEditor)
		    cons.newInstance(new Object[] {agent, dom});
	    // /\/: Special case to delete when possible:
	    if (ed instanceof JFrame)
		((JFrame)ed).setIconImage
		    (ix.iface.util.IconImage
		        .getIconImage("ip2-edit-icon.gif"));
	    return ed;
	}
	catch (Exception e) {
	    Debug.noteException(e);
	    Debug.noteln("Creating a SimpleDomainEditor instead.");
	    return new SimpleDomainEditor(agent, dom);
	}
    }

    public InternalDomainEditor getDomainEditor() {
	ensureDomainEditor();
	return domainEditor;
    }

    protected abstract void ensureDomainEditor();

}
// Issues:
// * Not clear that this should exist, or if it does that it should
//   have this functionality.

