/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Sun Jun  9 19:52:37 2002 by Jeff Dalton
 * Copyright: (c) 2002, AIAI, University of Edinburgh
 */

package ix.ip2;

import java.util.*;

import ix.icore.*;
import ix.util.*;
import ix.util.lisp.*;

public class QueryHandler extends ItemHandler {

    // Syntax is (query ?agentName ?parameters = ?resultPattern)

    Symbol S_QUERY = Symbol.intern("query"),
	   S_EQUALSIGN = Symbol.intern("=");

    public QueryHandler() {
	super("Query agent");
    }

    public boolean appliesTo(AgendaItem item) {
	LList pattern = item.getPattern();
	return pattern.length() == 5
	    && pattern.get(0) == S_QUERY
	    && pattern.get(3) == S_EQUALSIGN;
    }

    public void addHandlerActions(AgendaItem item) {
	// appliesTo will have already been called.
	String toName = item.getPattern().get(1).toString();
	item.addAction(new QueryAction(toName, item));
    }

    class QueryAction extends HandlerAction {

	String toName;
	AgendaItem item;
	boolean isReady = false;

	QueryAction(String toName, AgendaItem item) {
	    this.toName = toName;
	    this.item = item;
	    this.shortDescription = "Query " + toName;
	    computeStatus();
	}

	public boolean isReady() {
	    return isReady;
	}

	public void computeStatus() {
	    LList queryPattern = (LList)item.getPattern().get(2);
	    isReady = Variable.isFullyBound(queryPattern);
	}

	public void handle(AgendaItem item) {
	}

    }

}
