/* Author: Jeff Dalton
 * Updated: Thu May  9 01:02:22 2002 by Jeff Dalton
 * Copyright: (c) 2001 - 2002, AIAI, University of Edinburgh
 */

package ix.ip2;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.TableModelEvent;

import java.awt.BorderLayout;
import java.awt.Color;

import java.util.*;

import ix.icore.process.event.*;

import ix.iface.util.GridColumn;
import ix.util.*;
import ix.util.lisp.*;

public class StateViewTable extends JPanel implements StateViewer {

    protected Ip2 ip2;

    final int PAT_COL = 0, VAL_COL = 1;

    final int patternWidth = 30;
    final int valueWidth = 20;

    JTable table;
    StateModel model;

    List patternData = new ArrayList();

    /**
     * Constructs a viewer for the indicated agent.
     */
    public StateViewTable(Ip2 ip2) {
	super();
	this.ip2 = ip2;

	model = new StateModel();
	table = new JTable(model);
	table.getTableHeader().setReorderingAllowed(false);
 	table.getColumnModel().getColumn(PAT_COL)
 	    .setPreferredWidth(patternWidth * 10);
 	table.getColumnModel().getColumn(VAL_COL)
 	    .setPreferredWidth(valueWidth * 10);

	// JScrollPane scroll = new JScrollPane(table);
	// add(scroll);

	setLayout(new BorderLayout());
	add(table.getTableHeader(), BorderLayout.NORTH);
	add(table, BorderLayout.CENTER);

    }

    /**
     * Sets the viewer back to something approximating its initial state.
     */
    public void reset() {
	patternData.clear();
	// Tell the JTable that all row data has changed
	model.fireTableChanged(new TableModelEvent(model));
	invalidate();
    }

    /*
     * Methods in ProcessStatusListener interface
     */

    /** Ignored by this viewer. */
    public void statusUpdate(ProcessStatusEvent event) { }

    /** Ignored by this viewer. */
    public void newBindings(ProcessStatusEvent event, Map bindings) { }

    public void stateChange(ProcessStatusEvent event, Map delta) {
	for (Iterator i = delta.entrySet().iterator(); i.hasNext();) {
	    Map.Entry e = (Map.Entry)i.next();
	    LList pattern = (LList)e.getKey();
	    Object value = e.getValue();
	    recordNewValue(pattern, value);
	}
    }

    protected void recordNewValue(LList pattern, Object value) {
	Debug.noteln("State viewer sees " + pattern + " = " + value);

	PatternData dat = getPatternData(pattern);
	if (dat == null) {
	    // Add new row
	    dat = new PatternData(pattern, value);
	    patternData.add(dat);
	    dat.row = patternData.size() - 1;
	    model.fireTableRowsInserted(dat.row, dat.row);
	}
	else {
	    // Just change value
	    dat.value = value;
	    model.fireTableCellUpdated(dat.row, VAL_COL);
	}

    }

    /** The information we associate with a pattern. */
    class PatternData {
	LList pattern;
	Object value;
	int row;
	PatternData(LList pattern, Object value) {
	    this.pattern = pattern;
	    this.value = value;
	}
    }

    /** Find the data associated with a given pattern. */
    PatternData getPatternData(LList pattern) {
	for (Iterator i = patternData.iterator(); i.hasNext();) {
	    PatternData dat = (PatternData)i.next();
	    if (dat.pattern.equals(pattern))
		return dat;
	}
	return null;
    }

    /** Mediates between the data and the JTable. */
    class StateModel extends AbstractTableModel {

	final String[] columnName = {"Pattern", "Value"};

	StateModel() {
	    super();
	}

	public String getColumnName(int col) {
	    return columnName[col];
	}

	public int getColumnCount() {
	    return 2;
	}

	public int getRowCount() {
	    return patternData.size();
	}

	public Object getValueAt(int row, int col) {
	    PatternData dat = (PatternData)patternData.get(row);
	    switch(col) {
	    case PAT_COL:
		return Lisp.elementsToString(dat.pattern);
	    case VAL_COL:
		return dat.value;
	    }
	    throw new Error("Bogus column " + col);
	}

    }

}
