/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Fri Jun  7 02:46:44 2002 by Jeff Dalton
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 */

package ix.ispace;

import java.awt.Container;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.*;

import javax.swing.*;

import java.util.*;

import ix.icore.IXAgent;

import ix.iface.util.IconImage;
import ix.iface.util.GridColumn;
import ix.iface.util.CatchingActionListener;

import ix.ispace.event.*;

import ix.util.*;

public class IspaceTool implements ActionListener {

    IXAgent agent;
    ContactManager contactManager;

    JFrame frame;
    Container contentPane;

    RelationPanel relationPanel;

    public IspaceTool(IXAgent agent) {
	this.agent = agent;
	this.contactManager = agent.getContactManager();
	setUpFrame();
    }

    void setUpFrame() {		// or define a JFrame subclass??? /\/

	frame = new JFrame(agent.getAgentDisplayName() + " I-Space");
	contentPane = frame.getContentPane();

	// default is equiv to setLayout(new BorderLayout());
	frame.setIconImage(IconImage.getIconImage(this));
	frame.setJMenuBar(makeMenuBar());

	relationPanel = new RelationPanel();
	
	contentPane.add(new JScrollPane(relationPanel), BorderLayout.CENTER);
	contentPane.add(new ButtonPanel(), BorderLayout.SOUTH);

	contactManager.addContactListener(relationPanel);

	frame.pack();
	frame.setVisible(true);
    }

    public void setVisible(boolean v) {
	frame.setVisible(v);
    }

    /**
     * Creates the menu bar
     */
    protected JMenuBar makeMenuBar() {
	JMenuBar bar = new JMenuBar();

	JMenu fileMenu = new JMenu("File");
	bar.add(fileMenu);
	fileMenu.add(makeMenuItem("Close"));

	return bar;
    }

    // /\/: Must find a better way to do this rather than have
    // this method everywhere.
    protected JMenuItem makeMenuItem(String text) {
	JMenuItem item = new JMenuItem(text);
	item.addActionListener(CatchingActionListener.listener(this));
	return item;
    }

    /**
     * Action interpreter
     */
    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	Debug.noteln("I-Space Tool action:", command);
	if (command.equals("Close")) {
	    frame.setVisible(false);
	}
	else if (command.equals("Commit")) {
	    relationPanel.commitChanges();
	}
	else if (command.equals("Undo Uncommitted Changes")) {
	    relationPanel.resetChanges();
	}
	else
	    Debug.noteln("Nothing to do for", command);
    }


    class RelationPanel extends JPanel implements ContactListener {

	List rows = new LinkedList();

	GridColumn nameCol = new GridColumn("Agent");
	GridColumn relationCol = new GridColumn("Relation");

	Row newEntryRow;

	RelationPanel() {
	    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	    setBorder(BorderFactory.createTitledBorder("Relations"));
	    add(nameCol);
	    add(Box.createHorizontalStrut(5));
	    add(relationCol);

	    loadTable();
	}

//  	Row getRow(String name) {
//  	    for (Iterator ri = rows.iterator(); ri.hasNext();) {
//  		Row row = (Row)ri.next();
//  		if (row.name.equals(name))
//  		    return row;
//  	    }
//  	    return null;
//  	}

	void loadTable() {
	    List names = contactManager.getSortedNameList();
	    for (Iterator ni = names.iterator(); ni.hasNext();) {
		String name = (String)ni.next();
		Row row = new Row(name);
		row.addToTable(this);
	    }
	    newEntryRow = new Row(new JTextField(40), new RelationComboBox());
	    newEntryRow.addToTable(this);
	}

	void reloadTable() {
	    nameCol.reset();
	    relationCol.reset();
	    rows.clear();
	    loadTable();
	    invalidate();
	    frame.validate();
	    frame.pack();
	}

	void resetChanges() {
	    for (Iterator ri = rows.iterator(); ri.hasNext();) {
		Row row = (Row)ri.next();
		row.reset();
	    }
	}

	void commitChanges() {

	    // Relationship changes?
	    for (Iterator ri = rows.iterator(); ri.hasNext();) {
		Row row = (Row)ri.next();
		if (row == newEntryRow)
		    continue;
		AgentRelationship rel = row.relChoice.getSelectedRelation();
		if (rel == null)
		    contactManager.deleteAgent(row.name);
		else if (rel != row.rel) {
//  		    Debug.noteln("Changing " + row.name +
//  				 " from " + row.rel +
//  				 " to " + rel);
		    contactManager.changeRelationship(row.name, rel);
		}
	    }

	    // Add an agent?
	    String newName = newEntryRow.nameText.getText().trim();
	    if (!newName.equals("")) {
		AgentRelationship rel = 
		    newEntryRow.relChoice.getSelectedRelation();
		if (rel != null)		 // ie not a "Delete"
		    addNewEntry(newName, rel);
	    }

	    reloadTable();

	}

	void addNewEntry(String name, AgentRelationship rel) {
	    Debug.noteln("Adding agent " + name + " as " + rel);
	    if (contactManager.getAgentData(name) != null) {
		JOptionPane.showMessageDialog
		    (frame,
		     "Agent " + name + " already exists",
		     "Error",
		     JOptionPane.ERROR_MESSAGE);
		return;
	    }

	    contactManager.addAgent(name, rel);
	}

	// ContactListener methods

	public void contactChange(ContactEvent e) {
	    if (e.isNewContact()) {
		new Row(e.getNewData().getName())
		    .insertInTable(this);
	    }
	    // /\/: Else for now just reload.  Eventually need to figure
	    // out just what has to change and warn the user if it affects
	    // any uncommitted changes made in the panel.
	    reloadTable();
	}

    }

    class Row {

	String name;
	AgentRelationship rel;
	JTextField nameText;
	RelationComboBox relChoice;

	Row(String name) {
	    AgentData data = contactManager.getAgentData(name);

	    this.name = name;
	    this.rel = data.getRelationship();
	    this.nameText = new JTextField(name);
	    this.relChoice = new RelationComboBox();

	    nameText.setEditable(false);
	    relChoice.setRelation(data.getRelationship());
	}

	Row(JTextField nt, RelationComboBox rc) {
	    this.name = "";
	    this.rel = rc.getSelectedRelation();
	    this.nameText = nt;
	    this.relChoice = rc;
	}

	void reset() {
	    if (nameText.isEditable())
		nameText.setText(name);
	    relChoice.setRelation(rel);
	}

	void addToTable(RelationPanel p) {
	    p.rows.add(this);
	    p.nameCol.add(nameText);
	    p.relationCol.add(relChoice);
	}

	void insertInTable(RelationPanel p) {
	    ListIterator ri = p.rows.listIterator();
	    while (ri.hasNext()) {
		Row row = (Row)ri.next();
		// Insert before new-entry row (which is editable)
		// or before the first row with a lexicographically
		// later name.
		if (row.nameText.isEditable() 
		        || row.name.compareTo(name) > 0)
		    break;
	    }
	    int at = ri.nextIndex() - 1 + 1; // +1 because of col titles
	    p.rows.add(at, this);
	    p.nameCol.add(nameText, at);
	    p.relationCol.add(relChoice, at);
	    p.invalidate();
	    frame.validate();
	    frame.pack();
	}

    }

    class RelationComboBox extends JComboBox {

	public RelationComboBox() {
	    super();
	    for (Iterator i = AgentRelationship.values().iterator();
		 i.hasNext();) {
		AgentRelationship r = (AgentRelationship)i.next();
		addItem(Strings.capitalize(r.toString()));
	    }
	    addItem("Delete");
	    setSelectedItem
		(Strings.capitalize
		    (AgentRelationship.CONTACT.toString()));
	}

	public AgentRelationship getSelectedRelation() {
	    String name = (String)getSelectedItem();
	    return name == "Delete"
		? null
		: AgentRelationship.valueOf(name.toLowerCase());
	}

	public void setRelation(AgentRelationship r) {
	    setSelectedItem(Strings.capitalize(r.toString()));
	}

    }

    class ButtonPanel extends JPanel {

	ButtonPanel() {
	    // Stick with default flowlayout for now.
	    add(makeButton("Commit"));
	    add(makeButton("Undo Uncommitted Changes"));
	}

	protected JButton makeButton(String command) {
	    JButton b = new JButton(command);
	    b.addActionListener
		(CatchingActionListener.listener(IspaceTool.this));
	    return b;
	}

    }


}
