/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Thu May 23 00:11:51 2002 by Jeff Dalton
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 */

package ix.itest;

import javax.swing.*;
import java.awt.event.*;

import ix.iface.util.IconImage;
import ix.iface.util.CatchingActionListener;

import ix.ip2.PanelFrame;

import ix.util.*;

/**
 * The main frame of the I-TEST user interface.
 */
public class ItestFrame extends PanelFrame implements ActionListener {

    JMenu testMenu = new JMenu("Test", true); // tear-off "Test" menu

    /**
     * Creates the frame and adds its menu bar.
     */
    ItestFrame(String title) {
	super(title);
	setIconImage(IconImage.getIconImage(this));
	setJMenuBar(makeMenuBar());
    }

    /**
     * Creates the menu bar
     */
    protected JMenuBar makeMenuBar() {
	JMenuBar bar = new JMenuBar();

	JMenu fileMenu = new JMenu("File");
	bar.add(fileMenu);
	fileMenu.add(makeMenuItem("Exit"));

	bar.add(Box.createHorizontalGlue()); // let later items move right

	bar.add(testMenu);
	testMenu.setEnabled(false); 	// initially

	return bar;
    }

    // /\/: Must find a better way to do this rather than have
    // this method everywhere.
    protected JMenuItem makeMenuItem(String text) {
	JMenuItem item = new JMenuItem(text);
	item.addActionListener(CatchingActionListener.listener(this));
	return item;
    }

    // Required by PanelFrame /\/
    protected void ensureDomainEditor() {
	Debug.assert(false, "ItestFrame ensureDomainEditor() called");
    }

    /**
     * Action interpreter
     */
    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	Debug.noteln("I-TEST action:", command);
	if (command.equals("Exit")) {
	    if (Util.dialogConfirms(this, "Are you sure you want to exit?"))
		System.exit(0);		// should call agent exit /\/
	}
	else
	    Debug.noteln("Nothing to do for", command);
    }

    /**
     * Adds a test by adding a menu item to the "Test" menu.
     * The action listener will be wrapped in a CatchingActionListener.
     * All test additions should go directly or indirectly
     * through this method.
     *
     * @see ix.iface.util.CatchingActionListener
     */
    void addTest(String text, ActionListener listener) {
	JMenuItem item = new JMenuItem(text);
	item.addActionListener(CatchingActionListener.listener(listener));
	testMenu.setEnabled(true);
	testMenu.add(item);
    }

}

