/****************************************************************************
 * An editor/viewer panel for the domain itself (global information)
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 * ***was GlobalDomEdPanel
 *****************************************************************************
 */
package ix.iview;

import java.lang.Math;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;       
import javax.swing.tree.*;
//import java.awt.*;
import java.awt.Component;
import java.awt.event.*;
import ix.*;
import ix.util.Debug;
import ix.util.Util;
import ix.util.lisp.*;
import ix.icore.*;
import ix.icore.domain.*;
import ix.icore.domain.event.*;
import ix.iface.ui.*;
import ix.iface.ui.table.*;
import ix.iface.ui.util.*;
import ix.iface.domain.*;
import ix.iview.table.*;
import ix.iview.domain.*;
import ix.iview.domain.event.*;

/****************************************************************************
 * An editor/viewer panel for the domain itself (global information)
 *
 * Example code for using the GlobalFramePanel:
 *<PRE><code>
 *    ...
 *    domainPanel = new GlobalFramePanel(this);
 *    mainJPanel.add(domainPanel); 
 *    ...
 *</code></PRE>
 *****************************************************************************
 */
public class GlobalFramePanel extends AConstructFramePanel {

  //---------------------Fields-------------------------------------------

  protected EdMouseAdapter ml = new EdMouseAdapter();

  //---------------------Constructors----------------------------------------

  /**
   * Creates a construct frame panel with the given parent,
   * Creates all panel components, the mouse listener, and the relation editor.
   *
   * @param parent the DomainEditor frame that the panel belons to. Used 
   *   mainly to keep toggle buttons and menus in synch
   */    
  public GlobalFramePanel(){
    super();
  }
  public GlobalFramePanel(DomainEditor theParent){
    //make all things (some are made during initialisation)
    super(theParent);
    IFormModel model = new DomainFormModel();
    String[] fields = {"refinements"}; //, "annotations"
    model.setDisplayFields(fields);
    formPanel.setModel(model);
    UIDomain uid = parent.getUIDomain();
    setUIConstruct(uid);
    uid.addUIDomainListener((GlobalFormPanel)formPanel);
  }
 

  protected JPanel setupMainPanel() {
    formPanel = new GlobalFormPanel(this);
    Debug.noteln("made global form panel for DEP");
    return formPanel;
  }

  //---------------------Field access----------------------------------------

  public EdMouseAdapter getML() {
    return ml;
  }
  
  //---------------------Toggles----------------------------------------


  //---------------------Window building-------------------------------------


  //---------------------Public services--------------------------------------


  /**
   * Sets the construct's fields in their respective panels (fills the panels).
   */
  protected void setConstructFields() {
    
  }
  public List getAllConstructs() {
    List al = new LListCollector();
    al.add(parent.getUIDomain());
    return al;
  }
  public void newConstruct() {
    //parent.getUIDomain().setNewDomain(parent.getUIDomain().getDomain());
    notImp("Creating a new domain");
  }
  public UIObject makeNewConstruct() {
    return new UIDomain(parent);
  }
  public boolean checkConstruct(){
    notImp("Checking domains");
    return true;
  }
  /**
   * User request to save a domain.
   */
  public void saveConstruct(){
    noteConstruct();
    //parent.userSaveDomain();
  }

  /**
   * User request to revert an action to when it was last saved in 
   * the draft domain.
   */
  public void revertConstruct(){
    UIDomain uid = parent.getUIDomain();
    uid.loadFromDomain();
    setUIConstruct(uid);
  }
  /**
   * User request to revert an action to when it was last saved in 
   * the draft domain.
   */
  public void revertOConstruct(){
    UIDomain uid = parent.getUIDomain();
    uid.loadFromOriginal();
    setUIConstruct(uid);
  }

  protected UIObject noteConstruct() {
    //make sure there is a current construct
    UIDomain uid = parent.getUIDomain();
    //Debug.note("GFrP: saving domain name " + uid.getName());
    formPanel.saveToObject(uid);
    Debug.noteln(" new name:", uid.getName());
    uid.saveToDomain();
    setUIConstruct(uid);
    return uid;
  }

  public Named getNamedConstruct(String name) {
    Debug.noteln("Getting named domain " + name);
    if (parent.uiDomain == null) return null;
    else if (parent.uiDomain.getName().equals(name)) 
      return parent.uiDomain;
    else return null;
  }


  public void updatePanel() {
    setUIConstruct(parent.getUIDomain());
  }

  //default uses newConstruct which in this case calls userClearDomain (loops!)
  public void clear() {
    parent.getUIDomain().clear();
    updatePanel();
  }

  //-------------------Protected services--------------------------------------

  private void editRefinement() {
    //get the selection of the refinements list
    List refinements = getClearSelectedRefinements();
    if ((refinements != null) && !refinements.isEmpty()) {
      //just pick the first refinement
      Object refinement = refinements.get(0);
      //show the ActionEditorPanel via parent, setConstruct on the panel
      try { parent.setConstruct((IXObject)refinement); }
      catch (ClassCastException xe) { 
	try { parent.setUIConstruct((UIObject)refinement); }
	catch (ClassCastException ue) { 
	  try { 
	    UINodeSpec node = (UINodeSpec)refinement;
	    UIRefinement uir = new UIRefinement(getUIDomain());
	    uir.setPattern(node.getPattern());
	    uir.setName("Undefined");
	    parent.setUIConstruct(uir); 
	  }
	  catch (ClassCastException ne) { 
	    Debug.noteln("Domain Editor: Cannot edit things of class", 
			 refinement.getClass().getName());
	    //Debug.noteException(ne);
	  }
	}
      }
    }
  }
  private void addRefinement() {
    //show the ActionEditorPanel via parent, newConstruct on the panel
    parent.setConstruct(new Refinement());
  }
  
  private void deleteRefinement() {
    //get the selection of the refinements list
    List refinements = getClearSelectedRefinements();
    //delete the refinement from the UIDomain
    if (refinements != null) {
      for (Iterator i = refinements.iterator(); i.hasNext(); ){
	Object refinement = i.next();
	try { parent.getUIDomain().removeConstruct((UIObject)refinement); }
	catch (ClassCastException ce) { 
	  try { parent.getUIDomain().removeConstruct((IXObject)refinement); }
	  catch (ClassCastException e) { 
	    Debug.noteln("Domain Editor: Cannot delete things of class", 
			 refinement.getClass().getName());
	    //Debug.noteException(e);
	  }
	}
      }
    }
  }

  private List getClearSelectedRefinements() {
    IXEditorPanel refPanel = formPanel.getFieldBit("refinements");
    IXTreeTable table = (IXTreeTable) refPanel.getItemComponent();
    int[] rowNos = table.getSelectedRows();
    table.clearSelection();
    if (rowNos.length == 0) { //if selection is empty: complain
      String message = "Please select a refinement first.";
      JOptionPane.showMessageDialog(this,message);
      return null;
    } 
    else {
      List refinements = new ArrayList();
      ActionTreeTableModel model = (ActionTreeTableModel)table.getModel();
      for (int i = 0; i < rowNos.length; i++) {
	refinements.add(model.getRowObject(rowNos[i]));
      }
      return refinements;
    }
  }


  //----------------------Action services--------------------------------------
  


  public boolean frameActionPerformed(ActionEvent event){
    String command = event.getActionCommand();
    Debug.noteln("DomainFramePanel got frame action command", command);
    if (command == "newConstruct") newConstruct();
    else if (command == "editConstruct") {
      //get the construct name from the item text and show it
      JMenuItem item = (JMenuItem)event.getSource();
      String name = item.getText().trim();
      setUIConstruct((UIObject)getNamedConstruct(name));  
    }
    else if (command == "modify") saveConstruct();
    else if (command == "revert") revertConstruct();
    else if (command == "revertO") revertOConstruct();
    else if (command == "check") checkConstruct();
    else return false;
    return true;
  }
  /**
   * Wakes up the ActionListener with a user action.
   * This is called when a KeyStroke happens in which the ActionListener
   * registered its interest.
   */
  public void actionPerformed(ActionEvent ae){
    String command = ae.getActionCommand();
    Debug.noteln("Global Domain Panel got action command", command);
  }

  class EdMouseAdapter extends java.awt.event.MouseAdapter {
    public void mouseClicked(MouseEvent event) {
      Component component = event.getComponent();
      //component.requestFocus();
      //Debug.noteln("GlobalFramePanel: mouse from ", component.getClass());

      if (!AbstractButton.class.isAssignableFrom(component.getClass())) return;
      
      AbstractButton button = (AbstractButton) component;
      Debug.noteln("GFrP: Button command " + button.getText());
      if (button.getText() == "Add") 
	GlobalFramePanel.this.addRefinement();
      else if (button.getText() == "Delete") 
	GlobalFramePanel.this.deleteRefinement();
      else if (button.getText() == "Edit") 
	GlobalFramePanel.this.editRefinement();
    }
  }

  protected void notImp(String message) {
    UIUtil.notImplemented(this,message);
  }

}

/*Issues***************************************************************
 *
 *
/*Todos***************************************************************
 *
 * - 
 *  Wait for 
 *****************************************************************************
 */
