/****************************************************************************
 * A graphical editor/viewer panel for activity expansion structures
 *
 * @author Jussi Stader
 * @version 1.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iview.igraph;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import lt.monarch.graph.*;
import lt.monarch.graph.view.*;
import lt.monarch.graph.view.looks.*;
import lt.monarch.graph.view.utils.*;
import lt.monarch.graph.plugins.*;
import lt.monarch.graph.swing.*;

import ix.*;
import ix.util.*;
import ix.util.lisp.*;
import ix.icore.*;
import ix.icore.domain.*;
import ix.iface.ui.*;
import ix.iface.ui.util.*;
import ix.iface.domain.*;
import ix.iview.*;
import ix.iview.util.*;
import ix.iview.domain.*;


/****************************************************************************
 * A graphical editor/viewer panel for activity expansion structures
 *
 */
public class GExpansionEditor extends AConstructEditorPanel
implements ConstructEditing, ActionListener {

  private String name = "Expansion Editor";
  //private JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
  //private final JEditorPane notesEditor = new JEditorPane ();
  //protected final JPanel contentPanel = new JPanel (new BorderLayout ());
  private final ExpansionModel model = new ExpansionModel();
  protected ActionEditorPanel parent;
  protected NodeViewBase currentNode = null;
  private JPopupMenu nodePopup;
  private JMenu constructMenu;
  private JMenu expansionsMenu;
  private JTextField nameField;
  private JTextField patternField;
  private final GraphView graphView = new GraphView();
  private final MouseEventDistributor eventForwarder = 
                  new MouseEventDistributor();
  private final GraphPanel graphPanel = new GraphPanel(graphView);
  private final NodeSelectionManager selection = 
                  new NodeSelectionManager(model);


  public GExpansionEditor() {
    parent = null;
    Debug.error("Do not use constructor GExpansionEditor()" + 
		 " - give it a parent!");
  }

  public GExpansionEditor(ActionEditorPanel parent) {
    
    setLayout(new BorderLayout());
    //splitPane.setLeftComponent (this);
    //JScrollPane notesScrollPane = new JScrollPane (notesEditor);
    //splitPane.setRightComponent (notesScrollPane);
    //model = new ExpansionModel();
    model.setPanel(parent);
    this.parent = parent;


    //final GraphView graphView = new GraphView();
    graphView.setModel(model);
    //final GraphPanel graphPanel = new GraphPanel(graphView);

    //final NodeSelectionManager selection = new NodeSelectionManager(model);
    graphView.addPlugin(new SelectedNodeEmphasizer(selection));
    graphView.addPlugin(new NodeSelector(selection));

    LinkDragger linkDragger = new LinkDragger(model.getLinkStrategy());
    linkDragger.setInputModifierMask(0);
    graphView.addPlugin(linkDragger);

    graphView.addPlugin(new NodeBoxSelector(selection));
    graphView.addPlugin(new NodeDragger(selection));

    //final MouseEventDistributor eventForwarder = new MouseEventDistributor ();
    graphView.addPlugin(eventForwarder);
    graphView.setNodeViewFactory(new ExpansionNodeViewFactory());

    graphPanel.setBackground(new Color (240,240,240));
    graphPanel.setPreferredSize(new Dimension(500,500));
    //this.setPreferredSize(new Dimension(500,500));
    this.add(BorderLayout.CENTER, new JScrollPane(graphPanel));

    nodePopup = makePopup();

    makeToolbar();
    //splitPane.setDividerLocation(0.8);    
  }


  JToolBar toolBar = new JToolBar();
  ImageIcon newGNodeIcon = Util.resourceImageIcon("ide-add-node.gif");
  ImageIcon deleteGNodesIcon = Util.resourceImageIcon("ide-delete-nodes.gif");
  ImageIcon layoutIcon = Util.resourceImageIcon("ide-layout.gif");
  protected IXToolItem newGNodeItem =
    new IXToolItem(toolBar, this, "newGNode", "Add node", newGNodeIcon, 
		   "Add a new node to the graph");
  protected IXToolItem deleteGNodesItem =
    new IXToolItem(toolBar, this, "deleteGNodes", "Delete", deleteGNodesIcon, 
		   "Delete selected node(s)");
  protected IXToolItem layoutItem =
    new IXToolItem(toolBar, this, "layout", "Layout", layoutIcon,
		   "Layout graph");

  final Point newNodeLocation = new Point(100, 100);

  private void makeToolbar() {
    /*
    JToolBar toolbar = new JToolBar();
    JButton button;
    ImageIcon icon;
    icon = Util.resourceImageIcon("ide-add-node.gif");
    button = toolbar.add(new AbstractAction("Add node", icon) {
      public void actionPerformed(ActionEvent e) {
        Object node = model.addNewNode();
	if (node != null) {
	  NodeView view = (NodeView)graphView.nodeRegistry.getViewFor(node);
	  view.setPosition(newNodeLocation);
	  newNodeLocation.x += 20;
	  newNodeLocation.y += 20;
	}
      }
    });
    button.setToolTipText("Add node to the graph...");
    icon = Util.resourceImageIcon("ide-delete-nodes.gif");
    button = toolbar.add(new AbstractAction("Delete",icon) {
      public void actionPerformed(ActionEvent e) {
        Iterator nodes = selection.getSelectedNodes();
        while (nodes.hasNext()) 
          model.deleteNode((NodeSpec)nodes.next());
      }
    });
    button.setToolTipText("Delete selected node(s)");
    toolbar.addSeparator ();
    icon = Util.resourceImageIcon("ide-layout.gif");
    button = toolbar.add(new AbstractAction("Layout", icon) {
      public void actionPerformed(ActionEvent e) {
        graphView.layoutGraph();
      }
    });
    button.setToolTipText("Layout graph");
    toolbar.addSeparator ();
    nameField = new JTextField(10);
    patternField = new JTextField(10);
    MouseListener ml = new MouseAdapter() {};
    ThingEditorPanel nameBit = new ThingEditorPanel(ml, "Name", nameField);
    toolbar.add(nameBit);
    nameBit = new ThingEditorPanel(ml, "Pattern", patternField);
    toolbar.add(nameBit);

    this.add(BorderLayout.NORTH, toolbar);
    */
    
    newGNodeItem.show();
    deleteGNodesItem.show();
    toolBar.addSeparator();
    layoutItem.show();
    toolBar.addSeparator();

    nameField = new JTextField(10);
    patternField = new JTextField(10);
    MouseListener ml = new MouseAdapter() {};
    ThingEditorPanel nameBit = new ThingEditorPanel(ml, "Name", nameField);
    toolBar.add(nameBit);
    nameBit = new ThingEditorPanel(ml, "Pattern", patternField);
    toolBar.add(nameBit);

    this.add(BorderLayout.NORTH, toolBar);
  }


  private JPopupMenu makePopup() {
    JPopupMenu menu = new JPopupMenu();
    
    /*
    constructMenu = new JMenu("Set activity");
    populateConstructMenu();
    menu.add(constructMenu);
    */
    JMenuItem menuItem = new JMenuItem("Load graph");
    menuItem.addActionListener(GExpansionEditor.this);
    menu.add(menuItem);
    menu.addSeparator();
    menuItem = new JMenuItem("Properties");
    menuItem.addActionListener(GExpansionEditor.this);
    menu.add(menuItem);
    menu.addSeparator ();
    /*
    menuItem = new JMenuItem("Expansions");
    menuItem.addActionListener(GExpansionEditor.this);
    menu.add(menuItem);
    */
    expansionsMenu = new JMenu("Expansions");
    populateExpansionsMenu();
    menu.add(expansionsMenu);
    menuItem = new JMenuItem("New Expansion");
    menuItem.addActionListener(GExpansionEditor.this);
    menu.add(menuItem);
    return menu;
  }
  
  public void populateExpansionsMenu() {
    ArrayList exps = new ArrayList();
    if (currentNode != null) {
      NodeSpec node = (NodeSpec)currentNode.getModelNode();
      exps = parent.getAllExpansions(node);
    }
    UIUtil.populateMenu(this, expansionsMenu, "Expansions", exps);
  }


  public void setAction(UIRefinement uir) {
    uiConstruct = uir;
    loadFromObject();
  }

  public void layoutFromGraph(IGraph graph) {
    //Debug.noteln("GEE: layout from graph");
    Iterator mNodes = model.getNodes();
    while (mNodes.hasNext()) {
      Object mNode = mNodes.next();
      //Debug.noteln(" doing node " + mNode.toString());
      Point position = graph.getPosition(mNode);
      if (position != null) {
	//Debug.noteln(" Setting position " + position.toString());
	NodeView vNode = (NodeView)graphView.nodeRegistry.getViewFor(mNode);
	vNode.setPosition(position);
      }
    }
  }

  public void expandNode() {
    if (currentNode == null) return;  //this should not happen
    ExpansionModel.NodeComponent node = 
      (ExpansionModel.NodeComponent)currentNode.getModelNode();
    model.expandNode(node);
  }

  public void saveExpansionGraph(UIRefinement uir) {
    String name = uir.getName();
    //Debug.noteln("GEE: Saving expansion graph " + name);
    UIDomain uiDomain = parent.getUIDomain();
    if (uiDomain == null) return;
    IGraph graph = (IGraph)uiDomain.getNamedGraph(name);
    if (graph == null) {
      graph = new IGraph(name, graphView);
      uiDomain.addGraph(graph);
    }
    else graph.refreshGraph(graphView);
  }

  //-----------------------ConstructEditing things-----------------------

  
  /** Sets the current UI construct. */
  public void setUIConstruct(UIObject construct) {
    //if (construct.equals(uiConstruct)) return;
    setAction((UIRefinement)construct);
  }
  /** Sets the current UI construct. */
  public void setUIConstructOnly(UIObject construct) {
    //if (construct.equals(uiConstruct)) return;
    setAction((UIRefinement)construct);
  }

  /** Save the data from the panel into the given object. */
  public void saveToObject(Object object) {
    UIRefinement uir = (UIRefinement)object;
    if (IVUtil.checkCopyObject("activity", uir, nameField.getText().trim())) {
      //Debug.noteln("GEE: copying object");
      uir = new UIRefinement(parent.getUIDomain());
      saveToGivenObject(uir);
      parent.setUIConstructOnly(uir);
    }
    else saveToGivenObject(uir);
  }

  public void saveToGivenObject(UIRefinement uir) {
    //if (uir != null) Debug.noteln("GEE: saving object", uir.getName());
    //else Debug.noteln("GEE: saving null object");
    String name = nameField.getText().trim();
    String sPattern = patternField.getText().trim();
    uir.setName(name);
    uir.setPattern(Lisp.elementsFromString(sPattern));
    model.updateAction(uir);
    this.saveExpansionGraph(uir);
  }
  
  /** load the data for the panel from the current object. */
  public void loadFromObject() {
    UIRefinement uir = (UIRefinement)getUIConstruct();
    nameField.setText("");
    patternField.setText("");
    this.model.setAction(uir);
    if (uir != null) {
      nameField.setText(uir.getName());
      patternField.setText(UIUtil.listToDisplay(uir.getPattern()));
      UIDomain uiDomain = parent.getUIDomain();
      //Debug.noteln("GEE: getting graph from domain");
      IGraph graph = (IGraph)uiDomain.getNamedGraph(uir.getName());
      if (graph != null) layoutFromGraph(graph);
      else graphView.layoutGraph();
    }    
  }


  public String getNameValue() {
    return nameField.getText().trim();
  }



  //-----------------------Actions etc-----------------------------------------
  /**
   * Wakes up the ActionListener with a user action.
   * This is called when a KeyStroke happens in which the ActionListener
   * registered its interest.
   */
  public void actionPerformed(ActionEvent ae){
    String command = ae.getActionCommand();
    Debug.noteln("GEE got action command", command);
    if (command.equals("New Expansion")) expandNode();
    else if (command.equals("Expansions")) {
      //get the construct name from the item text and show it
      JMenuItem item = (JMenuItem)ae.getSource();
      String name = item.getText().trim();
      parent.setUIConstruct((UIObject)parent.getNamedConstruct(name)); 
    }
    else if (command.equals("newGNode")) {
      //Debug.noteln("Got add node");
      Object node = model.addNewNode();
      if (node != null) {
	NodeView view = (NodeView)graphView.nodeRegistry.getViewFor(node);
	view.setPosition(newNodeLocation);
	newNodeLocation.x += 20;
	newNodeLocation.y += 20;
      }
    }
    else if (command.equals("deleteGNodes")) {
      //Debug.noteln("Got delete");
      Iterator nodes = selection.getSelectedNodes();
      while (nodes.hasNext()) model.deleteNode((NodeSpec)nodes.next());
    }
    else if (command.equals("layout")) {
      //Debug.noteln("Got layout");
      graphView.layoutGraph();
    }
    //else if (command.equals("")) 
    else UIUtil.notImplemented(parent,command);
  }

  private class ExpansionNodeViewFactory implements NodeViewFactory {
    public NodeView createViewFor(Object o) {
      ExpansionModel.NodeComponent c = (ExpansionModel.NodeComponent)o;
      CompositeRectangularNode node = new 
	CompositeRectangularNode(graphView, graphPanel, c) {
	protected RectangularPin createPinFor(Object pin, String label) {
	  RectangularPin pinView = super.createPinFor(pin, label);
	  pinView.setBackground(Color.green);
	  return pinView;
	}
      };
      
      node.setOrientation(node.ORIENTATION_HORIZONTAL);
      node.setBackground(Color.white);
      node.setLabel(c.id + ": " + c.label);
      /*
      node.setLabel(c.label);
      Name id = c.id;
      if (id != null) node.setBottom(c.id.toString());
      */
      // create pin views for the node
      for (int i=0; i<c.inputPins.length; i++)
	node.addInputPin(c.inputPins[i], c.inputPins[i].label);
      for (int i=0; i<c.outputPins.length; i++)
	node.addOutputPin(c.outputPins[i], c.outputPins[i].label);
      // Right click on the node pops up a menu 
      final NodeViewBase targetNode = node;
      GExpansionEditor.this.eventForwarder.addMouseListener(node, new MouseAdapter() {
	public void mousePressed (MouseEvent e) {
	  if (!SwingUtilities.isRightMouseButton(e)) return;
	  GExpansionEditor.this.currentNode = targetNode;
	  //populateConstructMenu();//***should be done via popupMenuListener
	  populateExpansionsMenu();//***should be done via popupMenuListener
	  nodePopup.show(graphPanel, e.getX(), e.getY());
	}
      });
      
      return node;
    }
  }
  
}
