/****************************************************************************
 * A TreeTableModel that can display Refinement and Node objects.
 *
 * @author Jussi Stader
 * @version 2.0
 * Copyright: (c) 2001, AIAI, University of Edinburgh
 *
 *****************************************************************************
 */
package ix.iview.table;

import ix.util.*;
import ix.util.lisp.*;
import ix.icore.domain.*;
import ix.icore.domain.event.*;
import ix.icore.*;
import ix.iview.util.*;
import ix.iview.domain.*;
import ix.iview.domain.event.*;
import ix.iface.ui.table.*;
import ix.iface.ui.util.*;

import javax.swing.table.*;
import javax.swing.tree.*;
import java.util.*;


/****************************************************************************
 * A TreeTableModel that can display Refinement and Node objects.
 *****************************************************************************
 */
public class ActionTreeTableModel extends AbstractTreeTableModel 
  implements UIDomainListener
{
  protected UIDomain uiDomain;
  protected int PATTERN_COL = 1;

  public ActionTreeTableModel(UIDomain uiDomain) { 
    super();
    setColumnNames();
    setDomain(uiDomain);
  }

  public void setDomain(UIDomain domain) {
    uiDomain = domain;
    uiDomain.addUIDomainListener(this);
    List constructs = uiDomain.getAllRefinements();
    //Debug.noteln("ATTM: Refinements are:", UIUtil.listToDisplay(constructs));
    constructs = 
      (List)IVUtil.sortNamedCollection(new ArrayList(constructs));   
    //Debug.noteln(" after sort:", UIUtil.listToDisplay(constructs));
    setData(constructs);
  }

  /**
   * Makes an IXTreeTableNode from the given object.
   * If the given object is not a TreeNode already, uses the domain to get
   * the object's UIObject.
   */
  public IXTreeTableNode makeIXTreeTableNode(Object theNode){
    TreeNode tNode;
    if (theNode == null) tNode = null;
    else if (TreeNode.class.isInstance(theNode))
      tNode = (TreeNode)theNode;
    else if (IXObject.class.isInstance(theNode))
      tNode = (TreeNode)uiDomain.getUIObject((IXObject)theNode);
    else tNode = null;

    if (tNode == null) return null;
    else //make a new eNode using the tNode
      return new IXTNTreeTableNode(this, tNode);
  }



  /**
   * Gets the cell value for normal cells (only one - the pattern).
   * The name cell is done in the AbstractTreeTableModel method getTreeValueAt.
   * @return the value as an object, null for illegal columns.
   */
  public Object getCellValueAt(Object o, int columnIndex) {
    IXTNTreeTableNode iNode = (IXTNTreeTableNode)o;
    if (columnIndex == PATTERN_COL) {
      LList pattern = getPattern(iNode);
      return UIUtil.listToDisplay(pattern);
    }
    else return null;
  }

  /**
   * Gets the name string for the given row object.
   */
  public String getNameString(TreeNode node) {
    IXTNTreeTableNode iNode = (IXTNTreeTableNode)node;
    return ((Named)iNode.node).getName();
  }
  /**
   * Gets the strings that are to be used as column names.
   */
  void setColumnNames() {
    String[] names = {"Name", "Pattern"};
    super.setColumnNames(names);
  }


  protected LList getPattern(IXTNTreeTableNode iNode) {
    Object node = iNode.node;
    LList pattern = null;
    if (UIRefinement.class.isInstance(node))
      pattern = ((UIRefinement)node).getPattern();
    else if (Refinement.class.isInstance(node))
      pattern = ((Refinement)node).getPattern();
    else pattern = ((NodeSpec)node).getPattern();
    //Debug.noteln("ActTTM: pattern is", pattern.toString());
    return pattern;
  }


  public void reloadData() {
    //Debug.noteln("ATTM: reloadData");
    //(new Throwable()).printStackTrace();
    List constructs = uiDomain.getAllRefinements();
    //Debug.noteln("ATTM: Refinements are:", UIUtil.listToDisplay(constructs));
    constructs = 
      (List)IVUtil.sortNamedCollection(new ArrayList(constructs));   
    //Debug.noteln(" after sort:", UIUtil.listToDisplay(constructs));
    List data = new ArrayList(constructs);
    //Debug.noteln(" as arrayList:", UIUtil.listToDisplay(data));
    setData(data);
  }


  //----------------------------UIDomainListening things--------------------

  public void refinementAdded(RefinementEvent e) {
    if (!uiDomain.loading) addData(e.getRefinement());
  }
  public void refinementRemoved(RefinementEvent e) {
    removeData(e.getRefinement());
  }
  public void constructAdded(UIDomainEvent e) {
    if (!uiDomain.loading) addData(e.getObject());    
  }
  public void constructAdded(RefinementEvent e) {
    if (!uiDomain.loading) addData(e.getRefinement());    
  }
  public void constructRemoved(UIDomainEvent e) {
    removeData(e.getObject());
  }
  public void constructEdited(UIDomainEvent e) {
    fireObjectChanged(e.getObject());
  }
  public void domainCleared(UIDomainEvent e) {
    clearData();
    fireTableDataChanged();
  }
  //last two done in editor
  public void domainEdited(UIDomainEvent e) {
    //Debug.noteln("AcTTM: domain edited", e.getDomain().getName());
    //Debug.noteln(" tableModel is", this);
    setDomain(e.getDomain());
  }
  public void domainSet(UIDomainEvent e) {
    setDomain(e.getDomain());
  }


}
