/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Wed Mar 20 18:14:06 2002 by Jeff Dalton
 * Copyright: (c) 2002, AIAI, University of Edinburgh
 */

package ix.util.xml;

import java.util.*;

import org.jdom.Document;
import org.jdom.Element;

import ix.util.*;


/**
 * An XML Translator that reads two syntaxes but writes only one
 * and so can be used to convert to the output syntax.  This is
 * not parameterised in any flexible way; instead the class definition
 * must be edited to support whatever translation is desired at
 * the time.  Usually, it will extend the class that handles the
 * output syntax and have a translator for the alternative / old
 * syntax in a field.
 */

public class ConversionXMLTranslator extends DashSyntaxXMLTranslator {

    XMLTranslator oldSyntax = new JavaSyntaxXMLTranslator();

    public ConversionXMLTranslator() {
	super();
    }

    /**
     * Converts a JDOM Document to an object.
     */
    public Object objectFromDocument(Document doc) {
	if (isOldSyntax(doc))
	    return oldSyntax.objectFromDocument(doc);
	else
	    return objectFromElement(doc.getRootElement());
    }

    public boolean isOldSyntax(Document doc) {
	String name = doc.getRootElement().getName();
	return name.length() > 0
	    && Character.isUpperCase(name.charAt(0));
    }

}
